package cz.muni.fi.pa165.project.entity;

import cz.muni.fi.pa165.project.enums.Gender;
import cz.muni.fi.pa165.project.enums.UserRole;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.NoArgsConstructor;
import lombok.ToString;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.Enumerated;
import javax.persistence.FetchType;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import java.util.Collections;
import java.util.HashSet;
import java.util.Set;

/**
 * @author Aneta Čadová | 485025
 */
@AllArgsConstructor
@NoArgsConstructor
@EqualsAndHashCode(callSuper = false)
@Data
@Entity
@Table(name = "app_users")
public class AppUser extends BaseEntity<Long> {
    @Column(nullable = false, unique = true)
    private String email;

    @Column(nullable = false)
    private String passwordHash;

    @Column(nullable = false)
    private String firstName;

    @Column(nullable = false)
    private String lastName;

    @Column(nullable = false, unique = true)
    private String phoneNumber;
    @Enumerated
    private Gender gender;

    @Enumerated
    private UserRole role;

    @EqualsAndHashCode.Exclude
    @ToString.Exclude
    @OneToMany(mappedBy = "owner", fetch = FetchType.LAZY)
    private Set<House> houses = new HashSet<>();

    @EqualsAndHashCode.Exclude
    @ToString.Exclude
    @OneToMany(mappedBy = "owner", fetch = FetchType.LAZY)
    private Set<Device> devices = new HashSet<>();

    public AppUser(String email, String passwordHash, String firstName, String lastName, String phoneNumber, Gender gender, UserRole role) {
        this.email = email;
        this.passwordHash = passwordHash;
        this.firstName = firstName;
        this.lastName = lastName;
        this.phoneNumber = phoneNumber;
        this.gender = gender;
        this.role = role;
    }

    public Set<House> getHouses() {
        return Collections.unmodifiableSet(houses);
    }

    public Set<Device> getDevices() {
        return Collections.unmodifiableSet(devices);
    }
}
