package cz.muni.fi.pa165.project.entity;

import cz.muni.fi.pa165.project.enums.PowerStatus;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.NoArgsConstructor;
import lombok.ToString;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.Enumerated;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import java.util.Collections;
import java.util.HashSet;
import java.util.Set;

/**
 * @author Daniela Hostačná | 485432
 */
@AllArgsConstructor
@NoArgsConstructor
@EqualsAndHashCode(callSuper = false)
@Data
@Entity
@Table(name = "devices")
public class Device extends BaseEntity<Long> {
    @ManyToOne
    @JoinColumn(nullable = false)
    private AppUser owner;

    @Column(nullable = false)
    private String nickName;

    @Column(nullable = false)
    private String model;

    @Enumerated
    private PowerStatus powerStatus;

    @EqualsAndHashCode.Exclude
    @ToString.Exclude
    @OneToMany(mappedBy = "device", fetch = FetchType.LAZY)
    private Set<HouseDevice> houseDevices = new HashSet<>();

    @EqualsAndHashCode.Exclude
    @ToString.Exclude
    @OneToMany(mappedBy = "device", fetch = FetchType.LAZY)
    private Set<ConsumptionEvent> consumptionEvents = new HashSet<>();

    public Device(AppUser owner, String nickName, String model, PowerStatus powerStatus) {
        this.owner = owner;
        this.nickName = nickName;
        this.model = model;
        this.powerStatus = powerStatus;
    }

    public Set<HouseDevice> getHouseDevices() {
        return Collections.unmodifiableSet(houseDevices);
    }

    public Set<ConsumptionEvent> getConsumptionEvents() {
        return Collections.unmodifiableSet(consumptionEvents);
    }

    public void addConsumptionEvent(ConsumptionEvent event) {
        consumptionEvents.add(event);
    }
}
