package cz.muni.fi.pa165.project.entity;

import cz.muni.fi.pa165.project.enums.Gender;
import cz.muni.fi.pa165.project.enums.PowerStatus;
import cz.muni.fi.pa165.project.enums.UserRole;
import cz.muni.fi.pa165.project.repository.AppUserRepository;
import cz.muni.fi.pa165.project.repository.DeviceRepository;
import cz.muni.fi.pa165.project.repository.HouseRepository;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.ArrayList;
import java.util.List;

import static org.assertj.core.api.Assertions.assertThat;

/**
 * @author Tomáš Ondruško | 485418
 */
class DeviceTest extends BaseEntityTest {
    @Autowired
    private DeviceRepository deviceRepository;

    @Autowired
    private AppUserRepository appUserRepository;

    @Autowired
    private HouseRepository houseRepository;

    private AppUser appUser;
    private House house;
    private Device device;
    private Device otherDevice;

    @BeforeEach
    void beforeMethod() {
        appUser = new AppUser("email", "passwordHash", "firstName", "lastName", "phoneNumber", Gender.NON_BINARY, UserRole.APP_USER);

        house = new House(appUser, "name", "street", "city", "country", "zip");

        device = new Device(appUser, "nickName", "model", PowerStatus.ON);
        otherDevice = new Device(appUser, "otherNickName", "otherModel", PowerStatus.OFF);

        appUserRepository.save(appUser);
        houseRepository.save(house);
        deviceRepository.save(device);
        deviceRepository.save(otherDevice);
    }

    @Test
    void createAndPersistDeviceTest() {
        Device device = new Device(
                appUser,
                "nickName",
                "model",
                PowerStatus.ON
        );
        deviceRepository.save(device);
        assertThat(deviceRepository.findById(device.getId())).isPresent().get().isEqualTo(device);
    }

    @Test
    void findByIdTest() {
        assertThat(deviceRepository.findById(device.getId())).isPresent().get().isEqualTo(device);
    }

    @Test
    void existByIdTest() {
        assertThat(deviceRepository.existsById(device.getId())).isTrue();
    }

    @Test
    void findAllTest() {
        assertThat(deviceRepository.findAll()).containsExactly(device, otherDevice);
    }

    @Test
    void findAllByIdTest() {
        List<Long> deviceIdList = new ArrayList<>();
        deviceIdList.add(device.getId());
        deviceIdList.add(otherDevice.getId());
        assertThat(deviceRepository.findAllById(deviceIdList)).containsExactly(device, otherDevice);
    }

    @Test
    void saveTest() {
        var tmpDevice = deviceRepository.findById(otherDevice.getId());
        tmpDevice.orElseThrow().setModel("newModel");
        deviceRepository.save(tmpDevice.get());
        assertThat(deviceRepository.findById(tmpDevice.get().getId()).orElseThrow().getModel()).isEqualTo("newModel");
    }

    @Test
    void saveAllTest() {
        List<Long> deviceIdList = new ArrayList<>();
        deviceIdList.add(device.getId());
        deviceIdList.add(otherDevice.getId());

        var tmpDevice = deviceRepository.findAllById(deviceIdList);
        assertThat(tmpDevice).containsExactly(device, otherDevice);
        tmpDevice.forEach(h -> h.setModel("ultra"));
        deviceRepository.saveAll(tmpDevice);
        assertThat(deviceRepository.findById(device.getId()).orElseThrow().getModel()).isEqualTo("ultra");
        assertThat(deviceRepository.findById(otherDevice.getId()).orElseThrow().getModel()).isEqualTo("ultra");
    }

    @Test
    void countTest() {
        assertThat(deviceRepository.count()).isEqualTo(2);
    }

    @Test
    void deleteTest() {
        deviceRepository.delete(otherDevice);
        assertThat(deviceRepository.existsById(otherDevice.getId())).isFalse();
    }

    @Test
    void deleteByIdTest() {
        deviceRepository.deleteById(otherDevice.getId());
        assertThat(deviceRepository.existsById(otherDevice.getId())).isFalse();
    }

    @Test
    void deleteAllTest() {
        deviceRepository.deleteAll();
        assertThat(deviceRepository.findAll()).hasSize(0);
    }
}