package cz.muni.fi.pa165.project.entity;

import cz.muni.fi.pa165.project.enums.Gender;
import cz.muni.fi.pa165.project.enums.PowerStatus;
import cz.muni.fi.pa165.project.enums.UserRole;
import cz.muni.fi.pa165.project.repository.AppUserRepository;
import cz.muni.fi.pa165.project.repository.DeviceRepository;
import cz.muni.fi.pa165.project.repository.HouseDeviceRepository;
import cz.muni.fi.pa165.project.repository.HouseRepository;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;

import static org.assertj.core.api.Assertions.assertThat;

/**
 * @author Tomáš Ondruško | 485418
 */
@SuppressWarnings("CodeBlock2Expr")
class HouseDeviceTest extends BaseEntityTest {
    @Autowired
    private HouseDeviceRepository houseDeviceRepository;

    @Autowired
    private HouseRepository houseRepository;

    @Autowired
    private DeviceRepository deviceRepository;

    @Autowired
    private AppUserRepository appUserRepository;

    private AppUser appUser;
    private House house;
    private Device device;
    private HouseDevice houseDevice;
    private HouseDevice otherHouseDevice;

    @BeforeEach
    void beforeMethod() {
        appUser = new AppUser("email", "passwordHash", "firstName", "lastName", "phoneNumber", Gender.NON_BINARY, UserRole.APP_USER);

        house = new House(appUser, "name", "street", "city", "country", "zip");

        device = new Device(appUser, "nickName", "model", PowerStatus.ON);

        houseDevice = new HouseDevice(house, device, LocalDateTime.now(), LocalDateTime.now().plusSeconds(1));
        otherHouseDevice = new HouseDevice(house, device, LocalDateTime.now(), LocalDateTime.now().plusSeconds(1));

        appUserRepository.save(appUser);
        houseRepository.save(house);
        deviceRepository.save(device);
        houseDeviceRepository.save(houseDevice);
        houseDeviceRepository.save(otherHouseDevice);
    }

    @Test
    void createAndPersistHouseDeviceTest() {
        HouseDevice houseDevice = new HouseDevice(
                house,
                device,
                LocalDateTime.now(),
                LocalDateTime.now().plusSeconds(1)
        );
        houseDeviceRepository.save(houseDevice);
        assertThat(houseDeviceRepository.findById(houseDevice.getId())).isPresent().get().isEqualTo(houseDevice);
    }

    @Test
    void findByIdTest() {
        assertThat(houseDeviceRepository.findById(houseDevice.getId())).isPresent().get().isEqualTo(houseDevice);
    }

    @Test
    void existByIdTest() {
        assertThat(houseDeviceRepository.existsById(houseDevice.getId())).isTrue();
    }

    @Test
    void findAllTest() {
        assertThat(houseDeviceRepository.findAll()).containsExactly(houseDevice, otherHouseDevice);
    }

    @Test
    void findAllByIdTest() {
        List<Long> houseDeviceIdList = new ArrayList<>();
        houseDeviceIdList.add(houseDevice.getId());
        houseDeviceIdList.add(otherHouseDevice.getId());
        assertThat(houseDeviceRepository.findAllById(houseDeviceIdList)).containsExactly(houseDevice, otherHouseDevice);
    }

    @Test
    void countTest() {
        assertThat(houseDeviceRepository.count()).isEqualTo(2);
    }

    @Test
    void deleteTest() {
        houseDeviceRepository.delete(otherHouseDevice);
        assertThat(houseDeviceRepository.existsById(otherHouseDevice.getId())).isFalse();
    }

    @Test
    void deleteByIdTest() {
        houseDeviceRepository.deleteById(otherHouseDevice.getId());
        assertThat(houseDeviceRepository.existsById(otherHouseDevice.getId())).isFalse();
    }

    @Test
    void deleteAllTest() {
        houseDeviceRepository.deleteAll();
        assertThat(houseDeviceRepository.findAll()).hasSize(0);
    }
}