package cz.muni.fi.pa165.project.entity;

import cz.muni.fi.pa165.project.enums.Gender;
import cz.muni.fi.pa165.project.enums.UserRole;
import cz.muni.fi.pa165.project.repository.AppUserRepository;
import cz.muni.fi.pa165.project.repository.HouseRepository;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.ArrayList;
import java.util.List;

import static org.assertj.core.api.Assertions.assertThat;

/**
 * @author Aneta Čadová | 485025
 */
@SuppressWarnings("CodeBlock2Expr")
class HouseTest extends BaseEntityTest {
    @Autowired
    private HouseRepository houseRepository;

    @Autowired
    private AppUserRepository appUserRepository;

    private House house;
    private House sameNameHouse;
    private House differentNameHouse;
    private House updateHouse;
    private AppUser user;

    @BeforeEach
    void beforeMethod() {
        user = new AppUser("email", "hash", "firstName", "lastName", "123456789", Gender.FEMALE, UserRole.APP_USER);
        appUserRepository.save(user);

        house = new House(user, "FI MUNI", "Brno", "CZ", "Botanicka 68A", "60200");
        sameNameHouse = new House(user, "FI MUNI", "Brno", "CZ", "Street 1", "12345");
        differentNameHouse = new House(user, "house", "Brno", "CZ", "Street 1", "12345");
        updateHouse = new House(user, "house", "Olomouc", "CZ", "Street 1", "12345");

        houseRepository.save(house);
        houseRepository.save(sameNameHouse);
        houseRepository.save(differentNameHouse);
        houseRepository.save(updateHouse);
    }

    @Test
    void createAndPersistHouseTest() {
        House house = new House(user, "sampleHouse", "city", "country", "street", "12345");
        houseRepository.save(house);

        assertThat(houseRepository.findById(house.getId())).isPresent().get().isEqualTo(house);
    }

    @Test
    void findByNameTest() {
        assertThat(houseRepository.findByName("FI MUNI")).hasSize(2).contains(house, sameNameHouse);
    }

    @Test
    void findByIdTest() {
        assertThat(houseRepository.findById(house.getId())).isPresent().get().isEqualTo(house);
    }

    @Test
    void existByIdTest() {
        assertThat(houseRepository.existsById(house.getId())).isTrue();
    }

    @Test
    void findAllTest() {
        assertThat(houseRepository.findAll()).hasSize(4).contains(house, sameNameHouse, differentNameHouse, updateHouse);
    }

    @Test
    void findAllByIdTest() {
        List<Long> houseIdList = new ArrayList<>();
        houseIdList.add(house.getId());
        houseIdList.add(sameNameHouse.getId());
        assertThat(houseRepository.findAllById(houseIdList)).hasSize(2).contains(house, sameNameHouse);
    }

    @Test
    void saveTest() {
        var tmpHouse = houseRepository.findById(updateHouse.getId());
        assertThat(tmpHouse).isPresent();
        tmpHouse.get().setName("updatedName");
        houseRepository.save(tmpHouse.get());
        assertThat(houseRepository.findById(tmpHouse.get().getId()).orElseThrow().getName()).isEqualTo("updatedName");
    }

    @Test
    void saveAllTest() {
        List<Long> houseIdList = new ArrayList<>();
        houseIdList.add(house.getId());
        houseIdList.add(sameNameHouse.getId());

        var tmpHouses = houseRepository.findAllById(houseIdList);
        assertThat(tmpHouses).hasSize(2).contains(house, sameNameHouse);
        tmpHouses.forEach(h -> h.setName("updateAllHousesName"));
        houseRepository.saveAll(tmpHouses);
        assertThat(houseRepository.findById(house.getId()).orElseThrow().getName()).isEqualTo("updateAllHousesName");
        assertThat(houseRepository.findById(sameNameHouse.getId()).orElseThrow().getName()).isEqualTo("updateAllHousesName");
    }

    @Test
    void countTest() {
        assertThat(houseRepository.count()).isEqualTo(4);
    }

    @Test
    void deleteTest() {
        houseRepository.delete(sameNameHouse);
        assertThat(houseRepository.existsById(sameNameHouse.getId())).isFalse();
    }

    @Test
    void deleteByIdTest() {
        houseRepository.deleteById(updateHouse.getId());
        assertThat(houseRepository.existsById(updateHouse.getId())).isFalse();
    }

    @Test
    void deleteAllTest() {
        houseRepository.deleteAll();
        assertThat(houseRepository.findAll()).hasSize(0);
    }
}
