import axios from 'axios';
import React from 'react';
import BackButton from './BackButton';
import './Base.css';

interface IConsumptionEventsProps {

}

interface IConsumptionEventsState {
    consumptionEvents: IConsumptionEvent[];
    currentPage: number;
    recordsPerPage: number;
    totalPages: number;
}

interface IConsumptionEvent {
    id: string;
    timestamp: string;
    powerConsumption: string;
    name: string;
}

class ConsumptionEvents extends React.Component<IConsumptionEventsProps, IConsumptionEventsState> {
    constructor(props: IConsumptionEventsProps) {
        super(props);
        this.state = {
            consumptionEvents: [],
            currentPage: 1,
            recordsPerPage: 20,
            totalPages: 0
        }
    }

    async componentDidMount() {
        this.getConsumptionEventsByPagination(this.state.currentPage);
    }

    getConsumptionEventsByPagination(currentPage: number) {
        this.setState({ currentPage: currentPage });
        axios.get("/consumption_events?page=" + (currentPage - 1) + "&size=" + this.state.recordsPerPage,
            {
                headers: {
                    "authorization": `${sessionStorage.getItem("token")}`
                }
            }
        ).then(response => {
            this.setState({ consumptionEvents: response.data })
        });

        axios.get("/consumption_events/count",
            {
                headers: {
                    "authorization": `${sessionStorage.getItem("token")}`
                }
            }
        ).then(response => {
            var totalRecords = response.data
            this.setState({
                totalPages: Math.ceil(totalRecords / this.state.recordsPerPage)
            })
        });
    }

    showFirstPage = () => {
        this.getConsumptionEventsByPagination(1);
    }

    showPrevPage = () => {
        if (this.state.currentPage > 1) {
            this.getConsumptionEventsByPagination(this.state.currentPage - 1);
        }
    }

    showNextPage = () => {
        this.getConsumptionEventsByPagination(this.state.currentPage + 1);
    }

    showLastPage = () => {
        this.getConsumptionEventsByPagination(this.state.totalPages);
    }

    render() {
        const { consumptionEvents, currentPage, totalPages } = this.state;
        return (
            <div>
                {BackButton("/")}
                <div>
                    <h2 className="heading">Consumption events</h2>
                    <div className="list">
                        <>
                            {consumptionEvents.length === 0 ?
                                <>
                                    <p className="no_results">{":'("}</p>
                                    <p className="no_results">No consumption events found...</p>
                                </> :
                                <>
                                    <table>
                                        <thead>
                                            <tr>
                                                <th>ID</th>
                                                <th>Timestamp</th>
                                                <th>Power Consumption</th>
                                                <th>Name</th>
                                            </tr>
                                        </thead>
                                        <tbody>
                                            {consumptionEvents.map(event =>
                                                <tr key={event.id} className="basic_row">
                                                    <td>{event.id}</td>
                                                    <td>{event.timestamp}</td>
                                                    <td>{event.powerConsumption}</td>
                                                    <td>{event.name}</td>
                                                </tr>
                                            )}
                                        </tbody>
                                    </table>
                                    <div className="pagination_buttons" style={{ float: "left", fontFamily: "monospace", color: "#0275D8" }}>
                                        Page {currentPage} of {totalPages}
                                    </div>
                                    <div className="pagination_buttons" style={{ float: "right" }}>
                                        <nav>
                                            <ul className="pagination">
                                                <li className="page-item"><button className="page-link" disabled={currentPage === 1 ? true : false} onClick={this.showFirstPage}>{"<< First"}</button></li>
                                                <li className="page-item"><button className="page-link" disabled={currentPage === 1 ? true : false} onClick={this.showPrevPage}>{"< Previous"}</button></li>
                                                <li className="page-item"><button className="page-link" disabled={currentPage === totalPages ? true : false} onClick={this.showNextPage}>{"Next >"}</button></li>
                                                <li className="page-item"><button className="page-link" disabled={currentPage === totalPages ? true : false} onClick={this.showLastPage}>{"Last >>"}</button></li>
                                            </ul>
                                        </nav>
                                    </div>
                                </>
                            }
                        </>
                    </div>
                </div>
            </div>
        );
    }
}

export default ConsumptionEvents;
