import axios from 'axios';
import { useContext, useEffect, useRef, useState } from 'react';
import AuthContext from './context/AuthProvider';
const LOGIN_URL = '/login';

const Login = () => {
    const { setAuth }: any = useContext(AuthContext);
    const userRef = useRef<any>();
    const errRef = useRef<any>();

    const [user, setUser] = useState('');
    const [pwd, setPwd] = useState('');
    const [errMsg, setErrMsg] = useState('');
    const [success, setSuccess] = useState(false);

    useEffect(() => {
        userRef.current.focus();
    }, []);

    useEffect(() => {
        setErrMsg('');
    }, [user, pwd]);

    const handleSubmit = async (e: any) => {
        e.preventDefault();

        try {
            const response = await axios.post(LOGIN_URL,
                JSON.stringify({ email: user, password: pwd }),
                {
                    headers: { "Content-Type": "application/json" },
                    withCredentials: true
                }
            );
            const token = response.headers["authorization"];
            const authority = response.data["authorities"][0]["authority"];

            setAuth({ user, pwd, token });
            setUser('');
            setPwd('');
            setSuccess(true);

            // Store access token and authority
            sessionStorage.setItem("token", token);
            sessionStorage.setItem("authority", authority);

            window.location.href = "/pa165";
        } catch (err: any) {
            if (!err?.response) {
                setErrMsg("No Server Response");
            } else if (err.response?.status === 401) {
                setErrMsg("Unauthorized");
            } else {
                setErrMsg("Login Failed");
            }
            errRef.current.focus();
        }
    }

    return (
        <>
            {success ? (
                <p>Loading...</p>
            ) : (
                <section>
                    <p style={{ color: "red" }} ref={errRef} className={errMsg ? "errmsg" : "offscreen"} aria-live="assertive">{errMsg}</p>
                    <h2 className="heading">Log In</h2>
                    <form className="login_form" onSubmit={handleSubmit}>
                        <div className="login_fields">
                            <div className="login_labels">
                                <label id="username" htmlFor="username">Username: </label>
                                <input
                                    type="text"
                                    id="username"
                                    ref={userRef}
                                    autoComplete="off"
                                    onChange={(e) => setUser(e.target.value)}
                                    value={user}
                                    required
                                />
                            </div>
                            <div className="login_labels">
                                <label id="password" htmlFor="password">Password: </label>
                                <input
                                    type="password"
                                    id="password"
                                    onChange={(e) => setPwd(e.target.value)}
                                    value={pwd}
                                    required
                                />
                            </div>
                        </div>
                        <button className="login_button">Log In</button>
                    </form>
                </section>
            )}
        </>
    )
}

export default Login;
