import { BrowserRouter, Link, Route, Switch } from 'react-router-dom';
import './Base.css';
import ConsumptionEvents from './ConsumptionEvents';
import CreateDevice from './device/CreateDevice';
import DeviceDetail from './device/DeviceDetail';
import Devices from './device/Devices';
import EditDevice from './device/EditDevice';
import CreateHouse from './house/CreateHouse';
import EditHouse from './house/EditHouse';
import HouseDetail from './house/HouseDetail';
import Houses from './house/Houses';
import Login from './Login';
import ChangePassword from './profile/ChangePassword';
import EditProfile from './profile/EditProfile';
import Profile from './profile/Profile';
import Root from './Root';
import RootDistributor from './RootDistributor';

const PageNotFound = () => (
    <div>404! - <Link to="/">Home</Link></div>
)

const Unauthorized = () => (
    <div>401! - <Link to="/">Home</Link></div>
)

export default function Main() {
    var isDistributor = sessionStorage.getItem("authority") === "DISTRIBUTOR";
    return (
        <BrowserRouter basename="/pa165">
            <div className="main"><br />
                <Switch>
                    <Route exact path="/" component={isDistributor ? RootDistributor : Root} />

                    <Route exact path="/houses" component={!isDistributor ? Houses : Unauthorized} />
                    <Route exact path="/houses/:id" component={!isDistributor ? HouseDetail : Unauthorized} />
                    <Route exact path="/houses/:id/edit" component={!isDistributor ? EditHouse : Unauthorized} />
                    <Route exact path="/create-house" component={!isDistributor ? CreateHouse : Unauthorized} />
                    <Route exact path="/devices" component={!isDistributor ? Devices : Unauthorized} />
                    <Route exact path="/devices/:id" component={!isDistributor ? DeviceDetail : Unauthorized} />
                    <Route exact path="/devices/:id/edit" component={!isDistributor ? EditDevice : Unauthorized} />
                    <Route exact path="/create-device" component={!isDistributor ? CreateDevice : Unauthorized} />
                    <Route exact path="/profile" component={Profile} />
                    <Route exact path="/profile/edit" component={EditProfile} />
                    <Route exact path="/profile/change-password" component={ChangePassword} />
                    <Route exact path="/login" component={Login} />
                    <Route exact path="/consumption-events" component={isDistributor ? ConsumptionEvents : Unauthorized} />
                    <Route component={PageNotFound} />
                </Switch>
            </div>
        </BrowserRouter>
    );
}
