import axios from 'axios';
import React from 'react';
import { Redirect } from 'react-router-dom';
import { Tab, TabList, TabPanel, Tabs } from 'react-tabs';
import 'react-tabs/style/react-tabs.css';
import BackButton from '../BackButton';

interface IDeviceDetailProps {

}

interface IDeviceDetailState {
    exists: boolean;
    device: IDevice;
    assigned: boolean;
    houses: IHouse[];
    consumptionEvents: IConsumptionEvent[];
    currentPage: number;
    recordsPerPage: number;
    totalPages: number;
    selectedOption: string;
    orderBy: string;
}

interface IDevice {
    id: string;
    nickname: string;
    model: string;
    powerStatus: string;
    house: IDeviceHouse;
}

interface IConsumptionEvent {
    id: number;
    powerConsumption: string;
    timestamp: string;
    house: IDeviceHouse;
}

interface IDeviceHouse {
    id: string;
    name: string;
}

interface IHouse {
    id: string;
    city: string;
    country: string;
    name: string;
    street: string;
    zip: string;
    assignedDeviceCount: string;
}

export default class DeviceDetail extends React.Component<IDeviceDetailProps, IDeviceDetailState> {
    constructor(props: IDeviceDetailProps) {
        super(props);
        this.state = {
            exists: true,
            assigned: false,
            device: {
                id: "",
                nickname: "",
                model: "",
                powerStatus: "",
                house: {
                    id: "",
                    name: ""
                },
            },
            houses: [],
            consumptionEvents: [],
            currentPage: 1,
            recordsPerPage: 20,
            totalPages: 0,
            selectedOption: "id",
            orderBy: ",asc"
        }

        this.onValueChange = this.onValueChange.bind(this);
        this.onValueChangeOrder = this.onValueChangeOrder.bind(this);
        this.deleteDevice = this.deleteDevice.bind(this);
        this.handleAssign = this.handleAssign.bind(this);
        this.handleUnassign = this.handleUnassign.bind(this);
    }

    getId() {
        var id = window.location.pathname.split('/');
        return Number(id.at(id.length - 1));
    }

    getPage() {
        var id = window.location.href.split("?page=");
        return Number(id.at(1));
    }

    async componentDidMount() {
        var id = this.getId();
        axios.get("/devices/" + id,
            {
                headers: {
                    "authorization": `${sessionStorage.getItem("token")}`
                }
            }
        ).then(response => {
            var assigned = response.data["house"] !== null;
            this.setState({
                device: response.data,
                assigned: assigned
            });
        });

        axios.get("/me/houses",
            {
                headers: {
                    "authorization": `${sessionStorage.getItem("token")}`
                }
            }
        ).then(response => {
            const houses = response.data;
            this.setState({ houses: houses });
        });

        this.getConsumptionEventsByPagination(this.state.currentPage);
    }

    getConsumptionEventsByPagination(currentPage: number) {
        this.setState({ currentPage: currentPage });
        var id = this.getId();
        axios.get("/devices/" + id + "/consumption_events?page=" + (currentPage - 1) + "&size=" + this.state.recordsPerPage + "&sort=" + this.state.selectedOption + this.state.orderBy,
            {
                headers: {
                    "authorization": `${sessionStorage.getItem("token")}`
                }
            }
        ).then(response => {
            const consumptionEvents = response.data;
            this.setState({ consumptionEvents: consumptionEvents });
        });

        axios.get("/devices/" + id + "/consumption_events/count",
            {
                headers: {
                    "authorization": `${sessionStorage.getItem("token")}`
                }
            }
        ).then(response => {
            var totalRecords = response.data;
            this.setState({
                totalPages: Math.ceil(totalRecords / this.state.recordsPerPage)
            })
        });
    }

    async deleteDevice() {
        var result = window.confirm("Are you sure you want to delete this device?");
        const { device } = this.state;
        if (result) {
            axios.delete("/devices/" + device.id,
                {
                    headers: {
                        "authorization": `${sessionStorage.getItem("token")}`
                    }
                }
            ).then(() => {
                this.setState({
                    exists: false
                })
            });
            window.alert("A device has been deleted!");
        }
    }

    handleAssign(event: any) {
        var houseId = event.target.houseId.value;
        var deviceId = Number(this.getId());
        axios.post("/devices/" + deviceId + "/houses/" + houseId,
            {},
            {
                headers: {
                    "authorization": `${sessionStorage.getItem("token")}`
                }
            }
        );
    }

    async handleUnassign() {
        const { device } = this.state;
        var houseId = Number(device.house.id);
        var deviceId = Number(this.getId());
        axios.delete("/devices/" + deviceId + "/houses/" + houseId,
            {
                headers: {
                    "authorization": `${sessionStorage.getItem("token")}`
                }
            }
        );
        window.location.reload();
    }

    showFirstPage = () => {
        this.getConsumptionEventsByPagination(1);
    }

    showPrevPage = () => {
        if (this.state.currentPage > 1) {
            this.getConsumptionEventsByPagination(this.state.currentPage - 1);
        }
    }

    showNextPage = () => {
        this.getConsumptionEventsByPagination(this.state.currentPage + 1);
    }

    showLastPage = () => {
        this.getConsumptionEventsByPagination(this.state.totalPages);
    }

    onValueChange(event: any) {
        this.setState({
            selectedOption: event.target.value
        }, () => this.showFirstPage());
    }

    onValueChangeOrder(event: any) {
        this.setState({
            orderBy: event.target.value
        }, () => this.showFirstPage());
    }

    render() {
        const { exists, device, assigned, houses, consumptionEvents, currentPage, totalPages } = this.state;
        if (!exists) {
            return <Redirect to="/devices" />;
        }
        return (
            <div className="detail_layout">
                {BackButton("/devices")}
                <div className="detail">
                    <div className="detail_block house_info">
                        <div>
                            <h2>Device: {device.nickname}</h2>
                            <table>
                                <tbody>
                                    <tr className="basic_row">
                                        <td>ID</td>
                                        <td>{device.id}</td>
                                    </tr>
                                    <tr className="basic_row">
                                        <td>Model</td>
                                        <td>{device.model}</td>
                                    </tr>
                                    <tr className="basic_row">
                                        <td>Power status</td>
                                        <td>{device.powerStatus}</td>
                                    </tr>
                                </tbody>
                            </table>
                        </div>
                        <button className="stylized_button button_color edit_device_button" onClick={() => window.location.href = this.getId() + "/edit"}><i className="fa fa-edit"></i> Edit device</button>
                        {!assigned ?
                            <form onSubmit={this.handleAssign} className="assignment">
                                <button type="submit" className="stylized_button button_color assign_device_button"><i className="fa fa-check"></i> Assign</button>
                                <select name="houseId" id="houseId">
                                    {houses.map(house => {
                                        return <option key={house.id} value={house.id}>{house.name + " (ID: " + house.id + ")"}</option>
                                    })}
                                </select>
                            </form> :
                            <div className="assignment">
                                <button name="houseId" className="stylized_button button_color assign_device_button" onClick={this.handleUnassign}><i className="fa fa-close"></i> Unassign</button>
                                <div className="assigned_to_block">
                                    <p className="assigned_to">{"Assigned to:"}</p>
                                    <p className="assigned_to_2">{device.house.name + " (ID: " + device.house.id + ")"}</p>
                                </div>
                            </div>
                        }
                        <button className="stylized_button delete_device_button" onClick={this.deleteDevice}><i className="fa fa-close"></i> Delete device</button>
                    </div>
                    <div className="house_detail_block tabs">
                        <Tabs>
                            <TabList>
                                <Tab>Consumption events</Tab>
                            </TabList>

                            <TabPanel>
                                <div className="sort_data">
                                    <label>Sort by:</label>
                                    <select name="sortBy" id="sortBy" defaultValue={this.state.selectedOption} onChange={this.onValueChange}>
                                        <option value="id">ID</option>
                                        <option value="timestamp">Timestamp</option>
                                        <option value="powerConsumption">Power consumption</option>
                                    </select>
                                    <select name="order" id="order" defaultValue={this.state.orderBy} onChange={this.onValueChangeOrder}>
                                        <option value=",asc">Ascending</option>
                                        <option value=",desc">Descending</option>
                                    </select>
                                </div>
                                {consumptionEvents.length === 0 ?
                                    <>
                                        <p className="no_results">{":'("}</p>
                                        <p className="no_results">No consumption events found...</p>
                                    </> :
                                    <>
                                        <table>
                                            <thead>
                                                <tr>
                                                    <th>ID</th>
                                                    <th>Timestamp</th>
                                                    <th>Power Consumption</th>
                                                    <th>House ID</th>
                                                    <th>House name</th>
                                                </tr>
                                            </thead>
                                            <tbody>
                                                {consumptionEvents.map(event =>
                                                    <tr key={event.id} className="basic_row">
                                                        <td>{event.id}</td>
                                                        <td>{event.timestamp}</td>
                                                        <td>{event.powerConsumption}</td>
                                                        <td>{event.house.id}</td>
                                                        <td>{event.house.name}</td>
                                                    </tr>
                                                )}
                                            </tbody>
                                        </table>
                                        <div className="pagination_buttons" style={{ float: "left", fontFamily: "monospace", color: "#0275D8" }}>
                                            Page {currentPage} of {totalPages}
                                        </div>
                                        <div className="pagination_buttons" style={{ float: "right" }}>
                                            <nav>
                                                <ul className="pagination">
                                                    <li className="page-item"><button className="page-link" disabled={currentPage === 1 ? true : false} onClick={this.showFirstPage}>{"<< First"}</button></li>
                                                    <li className="page-item"><button className="page-link" disabled={currentPage === 1 ? true : false} onClick={this.showPrevPage}>{"< Previous"}</button></li>
                                                    <li className="page-item"><button className="page-link" disabled={currentPage === totalPages ? true : false} onClick={this.showNextPage}>{"Next >"}</button></li>
                                                    <li className="page-item"><button className="page-link" disabled={currentPage === totalPages ? true : false} onClick={this.showLastPage}>{"Last >>"}</button></li>
                                                </ul>
                                            </nav>
                                        </div>
                                    </>
                                }
                            </TabPanel>
                        </Tabs>
                    </div>
                </div>
            </div>
        );
    }
}
