import axios from 'axios';
import React from 'react';
import { Link } from 'react-router-dom';
import BackButton from '../BackButton';

interface IDevicesProps {

}

interface IDevicesState {
    devices: IDevice[];
    currentPage: number;
    recordsPerPage: number;
    totalPages: number;
    selectedOption: string;
    orderBy: string;
}

interface IDevice {
    id: number;
    nickname: string;
    model: string;
    powerStatus: string;
    house: IDeviceListHouse;
}

interface IDeviceListHouse {
    id: number;
    name: string;
}

export default class Devices extends React.Component<IDevicesProps, IDevicesState> {
    constructor(props: IDevicesProps) {
        super(props);
        this.state = {
            devices: [],
            currentPage: 1,
            recordsPerPage: 20,
            totalPages: 0,
            selectedOption: "id",
            orderBy: ",asc"
        }

        this.onValueChange = this.onValueChange.bind(this);
        this.onValueChangeOrder = this.onValueChangeOrder.bind(this);
    }

    componentDidMount() {
        this.getDevicesByPagination(this.state.currentPage);
    }

    async getDevicesByPagination(currentPage: number) {
        this.setState({ currentPage: currentPage });
        axios.get("/me/devices?page=" + (currentPage - 1) + "&size=" + this.state.recordsPerPage + "&sort=" + this.state.selectedOption + this.state.orderBy,
            {
                headers: {
                    "authorization": `${sessionStorage.getItem("token")}`
                }
            }
        ).then(response => {
            const devices = response.data;
            this.setState({ devices: devices });
        });

        axios.get("/me/devices/count",
            {
                headers: {
                    "authorization": `${sessionStorage.getItem("token")}`
                }
            }
        ).then(response => {
            var totalRecords = response.data;
            this.setState({
                totalPages: Math.ceil(totalRecords / this.state.recordsPerPage)
            })
        });
    }

    showFirstPage = () => {
        this.getDevicesByPagination(1);
    }

    showPrevPage = () => {
        if (this.state.currentPage > 1) {
            this.getDevicesByPagination(this.state.currentPage - 1);
        }
    }

    showNextPage = () => {
        this.getDevicesByPagination(this.state.currentPage + 1);
    }

    showLastPage = () => {
        this.getDevicesByPagination(this.state.totalPages);
    }

    onValueChange(event: any) {
        this.setState({
            selectedOption: event.target.value
        }, () => this.showFirstPage());
    }

    onValueChangeOrder(event: any) {
        this.setState({
            orderBy: event.target.value
        }, () => this.showFirstPage());
    }

    render() {
        const { devices, currentPage, totalPages } = this.state;
        return (
            <div className="layout">
                {BackButton("/")}
                <div>
                    <h2 className="heading">My devices</h2>
                    <div className="sort_data">
                        <label>Sort by:</label>
                        <select name="sortBy" id="sortBy" defaultValue={this.state.selectedOption} onChange={this.onValueChange}>
                            <option value="id">ID</option>
                            <option value="nickName">Nickname</option>
                            <option value="model">Model</option>
                            <option value="powerStatus">Power status</option>
                        </select>
                        <select name="order" id="order" defaultValue={this.state.orderBy} onChange={this.onValueChangeOrder}>
                            <option value=",asc">Ascending</option>
                            <option value=",desc">Descending</option>
                        </select>
                    </div>
                    <div className="list">
                        <>
                            {devices.length === 0 ?
                                <>
                                    <p className="no_results">{":'("}</p>
                                    <p className="no_results">No devices found...</p>
                                </> :
                                <>
                                    <table>
                                        <thead>
                                            <tr>
                                                <th>ID</th>
                                                <th>Nickname</th>
                                                <th>Model</th>
                                                <th>Power Status</th>
                                                <th>House ID</th>
                                                <th>House name</th>
                                            </tr>
                                        </thead>
                                        <tbody>
                                            {devices.map(device =>
                                                <tr key={device.id} className="tbl_row" onClick={() => window.location.href = "devices/" + device.id + "?page=" + (this.state.currentPage - 1)}>
                                                    <td >{device.id}</td>
                                                    <td>{device.nickname}</td>
                                                    <td>{device.model}</td>
                                                    <td>{device.powerStatus}</td>
                                                    <td>{device.house === null ? "-" : device.house.id}</td>
                                                    <td>{device.house === null ? "" : device.house.name}</td>
                                                </tr>
                                            )}
                                        </tbody>
                                    </table>
                                    <div className="pagination_buttons" style={{ float: "left", fontFamily: "monospace", color: "#0275D8" }}>
                                        Page {currentPage} of {totalPages}
                                    </div>
                                    <div className="pagination_buttons" style={{ float: "right" }}>
                                        <nav>
                                            <ul className="pagination">
                                                <li className="page-item"><button className="page-link" disabled={currentPage === 1 ? true : false} onClick={this.showFirstPage}>{"<< First"}</button></li>
                                                <li className="page-item"><button className="page-link" disabled={currentPage === 1 ? true : false} onClick={this.showPrevPage}>{"< Previous"}</button></li>
                                                <li className="page-item"><button className="page-link" disabled={currentPage === totalPages ? true : false} onClick={this.showNextPage}>{"Next >"}</button></li>
                                                <li className="page-item"><button className="page-link" disabled={currentPage === totalPages ? true : false} onClick={this.showLastPage}>{"Last >>"}</button></li>
                                            </ul>
                                        </nav>
                                    </div>
                                </>
                            }
                        </>
                    </div>
                </div>
                <div className="add_button">
                    <Link to="/create-device">
                        <button className="stylized_button button_color"><i className="fa fa-plus"></i> Add new device</button>
                    </Link>
                </div>
            </div>
        );
    }
}
