import axios from "axios";
import React from "react";
import BackButton from "../BackButton";

interface IEditDeviceProps {

}

interface IEditDeviceState {
    changed: boolean;
    device: IDevice;
    selectedOption: string;
}

interface IDevice {
    id: string;
    nickname: string;
    model: string;
    powerStatus: string;
    house: IDeviceHouse;
}

interface IDeviceHouse {
    id: string;
    name: string;
}

export default class EditDevice extends React.Component<IEditDeviceProps, IEditDeviceState> {
    constructor(props: IEditDeviceProps) {
        super(props);
        this.state = {
            changed: false,
            device: {
                id: "",
                nickname: "",
                model: "",
                powerStatus: "",
                house: {
                    id: "",
                    name: ""
                }
            },
            selectedOption: ""
        }

        this.onValueChange = this.onValueChange.bind(this);
        this.handleSubmit = this.handleSubmit.bind(this);
    }

    getId() {
        var id = window.location.pathname.split('/');
        return Number(id.at(id.length - 2));
    }

    componentDidMount() {
        this.getDeviceData();
    }

    getDeviceData() {
        axios.get("/devices/" + this.getId(),
            {
                headers: {
                    "authorization": `${sessionStorage.getItem("token")}`
                }
            }
        ).then(response => {
            const deviceData = response.data;
            this.setState({
                device: deviceData,
                selectedOption: response.data["powerStatus"]
            });
        });
    }

    handleSubmit(event: any) {
        axios.put("/devices/" + this.getId(),
            JSON.stringify(
                {
                    nickname: event.target.nickname.value,
                    model: event.target.model.value,
                    powerStatus: this.state.selectedOption
                }
            ),
            {
                headers: {
                    "Content-Type": "application/json",
                    "authorization": `${sessionStorage.getItem("token")}`
                }
            }
        ).then(() => {
            this.setState({
                changed: true
            })
        });
    }

    onValueChange(event: any) {
        this.setState({
            selectedOption: event.target.value
        });
    }

    render() {
        const { changed, device } = this.state;
        if (changed) {
            window.location.href = window.location.pathname.split("/edit")[0];
        }
        return (
            <div>
                {BackButton("/devices/" + this.getId())}
                <h2>Edit device</h2>
                <form onSubmit={this.handleSubmit} className="custom_form">
                    <label>Nickname: </label>
                    <input type="text" name="nickname" defaultValue={device.nickname} required /><br />
                    <label>Model: </label>
                    <input type="text" name="model" defaultValue={device.model} required /><br />
                    <div className="power_status_block">
                        <label className="power_status">Power status: </label>
                        <div className="block">
                            <div>
                                <input type="radio" name="powerStatus" value="ON" checked={this.state.selectedOption === "ON"} onChange={this.onValueChange} />
                                <label>ON</label>
                            </div>
                            <div>
                                <input type="radio" name="powerStatus" value="OFF" checked={this.state.selectedOption === "OFF"} onChange={this.onValueChange} />
                                <label>OFF</label>
                            </div>
                        </div>
                    </div>
                    <input className="stylized_button button_color" type="submit" value="Update" id="submit" />
                </form>
            </div>
        );
    }
}
