import axios from "axios";
import React from "react";
import BackButton from "../BackButton";

interface IEditHouseProps {

}

interface IEditHouseState {
    changed: boolean;
    house: IHouse;
}

interface IHouse {
    name: string;
    street: string;
    city: string;
    country: string;
    zip: string;
}

export default class EditHouse extends React.Component<IEditHouseProps, IEditHouseState> {
    constructor(props: IEditHouseProps) {
        super(props);
        this.state = {
            changed: false,
            house: {
                name: "",
                street: "",
                city: "",
                country: "",
                zip: ""
            }
        }

        this.handleSubmit = this.handleSubmit.bind(this);
    }

    getId() {
        var id = window.location.pathname.split('/');
        return id.at(id.length - 2);
    }

    componentDidMount() {
        this.getHouseData();
    }

    getHouseData() {
        axios.get("/houses/" + this.getId(),
            {
                headers: {
                    "authorization": `${sessionStorage.getItem("token")}`
                }
            }
        ).then(response => {
            this.setState({ house: response.data });
        });
    }

    handleSubmit(event: any) {
        axios.put("/houses/" + this.getId(),
            JSON.stringify(
                {
                    name: event.target.name.value,
                    street: event.target.street.value,
                    city: event.target.city.value,
                    country: event.target.country.value,
                    zipCode: event.target.zipCode.value
                }
            ),
            {
                headers: {
                    "Content-Type": "application/json",
                    "authorization": `${sessionStorage.getItem("token")}`
                }
            }
        ).then(() => {
            this.setState({
                changed: true
            })
        });
    }

    render() {
        const { changed, house } = this.state;
        if (changed) {
            window.location.href = window.location.pathname.split("/edit")[0];
        }
        return (
            <div>
                {BackButton("/houses/" + this.getId())}
                <h2>Edit house</h2>
                <form onSubmit={this.handleSubmit} className="custom_form">
                    <label>Name: </label>
                    <input type="text" name="name" defaultValue={house.name} required /><br />
                    <label>Street: </label>
                    <input type="text" name="street" defaultValue={house.street} required /><br />
                    <label>City: </label>
                    <input type="text" name="city" defaultValue={house.city} required /><br />
                    <label>Country: </label>
                    <input type="text" name="country" defaultValue={house.country} required /><br />
                    <label>ZIP: </label>
                    <input type="text" name="zipCode" defaultValue={house.zip} required /><br />
                    <input className="stylized_button button_color" type="submit" value="Update" id="submit" />
                </form>
            </div>
        );
    }
}
