import axios from 'axios';
import React from 'react';
import { Redirect } from 'react-router-dom';
import { Tab, TabList, TabPanel, Tabs } from 'react-tabs';
import 'react-tabs/style/react-tabs.css';
import BackButton from '../BackButton';

interface IHouseDetailProps {

}

interface IHouseDetailState {
    exists: boolean;
    house: IHouse;

    devices: IDevice[];
    currentPageDevices: number;
    recordsPerPageDevices: number;
    totalPagesDevices: number;
    selectedOptionForDevices: string;
    orderByForDevices: string;

    consumptionEvents: IConsumptionEvent[];
    currentPageConsumptionEvents: number;
    recordsPerPageConsumptionEvents: number;
    totalPagesConsumptionEvents: number;
    selectedOptionForConsumptionEvents: string;
    orderByForConsumptionEvents: string;
}

interface IHouse {
    id: string;
    name: string;
    street: string;
    city: string;
    country: string;
    zip: string;
    assignedDeviceCount: string;
}

interface IDevice {
    id: number;
    nickname: string;
    model: string;
    powerStatus: string;
}

interface IConsumptionEvent {
    id: number;
    powerConsumption: string;
    timestamp: string;
    device: IConsumptionEventDevice;
}

interface IConsumptionEventDevice {
    id: number;
    nickname: string;
}

export default class HouseDetail extends React.Component<IHouseDetailProps, IHouseDetailState> {
    constructor(props: IHouseDetailProps) {
        super(props);
        this.state = {
            exists: true,
            house: {
                id: "",
                name: "",
                street: "",
                city: "",
                country: "",
                zip: "",
                assignedDeviceCount: ""
            },
            devices: [],
            currentPageDevices: 1,
            recordsPerPageDevices: 20,
            totalPagesDevices: 0,
            selectedOptionForDevices: "id",
            orderByForDevices: ",asc",
            consumptionEvents: [],
            currentPageConsumptionEvents: 1,
            recordsPerPageConsumptionEvents: 20,
            totalPagesConsumptionEvents: 0,
            selectedOptionForConsumptionEvents: "id",
            orderByForConsumptionEvents: ",asc"
        }

        this.onValueChangeDevices = this.onValueChangeDevices.bind(this);
        this.onValueChangeOrderDevices = this.onValueChangeOrderDevices.bind(this);
        this.onValueChangeConsumptionEvents = this.onValueChangeConsumptionEvents.bind(this);
        this.onValueChangeOrderConsumptionEvents = this.onValueChangeOrderConsumptionEvents.bind(this);
        this.shutdown = this.shutdown.bind(this);
        this.deleteHouse = this.deleteHouse.bind(this);
    }

    getId() {
        var id = window.location.pathname.split('/');
        return Number(id.at(id.length - 1));
    }

    getPage() {
        var id = window.location.href.split("?page=");
        return Number(id.at(1));
    }

    async componentDidMount() {
        var id = Number(this.getId());
        axios.get("/me/houses?page=" + this.getPage(),
            {
                headers: {
                    "authorization": `${sessionStorage.getItem("token")}`
                }
            }
        ).then(response => {
            const houses = response.data;
            const house = houses.filter((house: any) => house.id === id)[0];
            this.setState({ house: house });
        });

        this.getHouseDevicesByPagination(this.state.currentPageDevices);
        this.getHouseConsumptionEventsByPagination(this.state.currentPageConsumptionEvents);
    }

    getHouseDevicesByPagination(currentPageDevices: number) {
        this.setState({ currentPageDevices: currentPageDevices });
        var id = Number(this.getId());
        axios.get("/houses/" + id + "/devices?page=" + (currentPageDevices - 1) + "&size=" + this.state.recordsPerPageDevices + "&sort=" + this.state.selectedOptionForDevices + this.state.orderByForDevices,
            {
                headers: {
                    "authorization": `${sessionStorage.getItem("token")}`
                }
            }
        ).then(response => {
            const devices = response.data;
            this.setState({ devices: devices });
        });

        axios.get("/houses/" + id + "/devices/count",
            {
                headers: {
                    "authorization": `${sessionStorage.getItem("token")}`
                }
            }
        ).then(response => {
            var totalRecordsDevices = response.data;
            this.setState({
                totalPagesDevices: Math.ceil(totalRecordsDevices / this.state.recordsPerPageDevices)
            })
        });
    }

    getHouseConsumptionEventsByPagination(currentPageConsumptionEvents: number) {
        this.setState({ currentPageConsumptionEvents: currentPageConsumptionEvents });
        var id = Number(this.getId());
        axios.get("/houses/" + id + "/consumption_events?page=" + (currentPageConsumptionEvents - 1) + "&size=" + this.state.recordsPerPageConsumptionEvents + "&sort=" + this.state.selectedOptionForConsumptionEvents + this.state.orderByForConsumptionEvents,
            {
                headers: {
                    "authorization": `${sessionStorage.getItem("token")}`
                }
            }
        ).then(response => {
            const consumptionEvents = response.data;
            this.setState({ consumptionEvents: consumptionEvents });
        });

        axios.get("/houses/" + id + "/consumption_events/count",
            {
                headers: {
                    "authorization": `${sessionStorage.getItem("token")}`
                }
            }
        ).then(response => {
            var totalRecordsConsumptionEvents = response.data;
            this.setState({
                totalPagesConsumptionEvents: Math.ceil(totalRecordsConsumptionEvents / this.state.recordsPerPageConsumptionEvents)
            })
        });
    }

    async deleteHouse() {
        var result = window.confirm("Are you sure you want to delete this house?");
        const { house } = this.state;
        if (result) {
            axios.delete("/houses/" + house.id,
                {
                    headers: {
                        "authorization": `${sessionStorage.getItem("token")}`
                    }
                }
            ).then(() => {
                this.setState({
                    exists: false
                })
            });
            window.alert("A house has been deleted!");
        }
    }

    async shutdown() {
        var result = window.confirm("Are you sure you want to shutdown this house?");
        const { house } = this.state;
        if (result) {
            axios.put("/houses/" + house.id + "/shut_down",
                {},
                {
                    headers: {
                        "authorization": `${sessionStorage.getItem("token")}`
                    }
                }
            );
        }
        window.location.reload();
    }

    showFirstPageDevices = () => {
        this.getHouseDevicesByPagination(1);
    }

    showPrevPageDevices = () => {
        if (this.state.currentPageDevices > 1) {
            this.getHouseDevicesByPagination(this.state.currentPageDevices - 1);
        }
    }

    showNextPageDevices = () => {
        this.getHouseDevicesByPagination(this.state.currentPageDevices + 1);
    }

    showLastPageDevices = () => {
        this.getHouseDevicesByPagination(this.state.currentPageDevices);
    }

    showFirstPageConsumptionEvents = () => {
        this.getHouseConsumptionEventsByPagination(1);
    }

    showPrevPageConsumptionEvents = () => {
        if (this.state.currentPageConsumptionEvents > 1) {
            this.getHouseConsumptionEventsByPagination(this.state.currentPageConsumptionEvents - 1);
        }
    }

    showNextPageConsumptionEvents = () => {
        this.getHouseConsumptionEventsByPagination(this.state.currentPageConsumptionEvents + 1);
    }

    showLastPageConsumptionEvents = () => {
        this.getHouseConsumptionEventsByPagination(this.state.currentPageConsumptionEvents);
    }

    onValueChangeDevices(event: any) {
        this.setState({
            selectedOptionForDevices: event.target.value
        }, () => this.showFirstPageDevices());
    }

    onValueChangeOrderDevices(event: any) {
        this.setState({
            orderByForDevices: event.target.value
        }, () => this.showFirstPageDevices());
    }

    onValueChangeConsumptionEvents(event: any) {
        this.setState({
            selectedOptionForConsumptionEvents: event.target.value
        }, () => this.showFirstPageConsumptionEvents());
    }

    onValueChangeOrderConsumptionEvents(event: any) {
        this.setState({
            orderByForConsumptionEvents: event.target.value
        }, () => this.showFirstPageConsumptionEvents());
    }

    render() {
        const { exists, house, devices, consumptionEvents, currentPageDevices, totalPagesDevices, currentPageConsumptionEvents, totalPagesConsumptionEvents } = this.state;
        if (!exists) {
            return <Redirect to="/houses" />;
        }
        return (
            <div className="detail_layout">
                {BackButton("/houses")}
                <div className="detail">
                    <div className="detail_block house_info">
                        <h2>House: {house.name}</h2>
                        <table>
                            <tbody>
                                <tr className="basic_row">
                                    <td>ID</td>
                                    <td>{house.id}</td>
                                </tr>
                                <tr className="basic_row">
                                    <td>Street</td>
                                    <td>{house.street}</td>
                                </tr>
                                <tr className="basic_row">
                                    <td>City</td>
                                    <td>{house.city}</td>
                                </tr>
                                <tr className="basic_row">
                                    <td>Country</td>
                                    <td>{house.country}</td>
                                </tr>
                                <tr className="basic_row">
                                    <td>ZIP</td>
                                    <td>{house.zip}</td>
                                </tr>
                                <tr className="basic_row">
                                    <td>Assigned devices</td>
                                    <td>{house.assignedDeviceCount}</td>
                                </tr>
                            </tbody>
                        </table>
                        <button className="stylized_button button_color edit_house_button" onClick={() => window.location.href = this.getId() + "/edit"}><i className="fa fa-edit"></i> Edit house</button>
                        <button className="stylized_button shutdown_house_button" onClick={this.shutdown} title="Turn off all devices"><i className="fa fa-power-off"></i> Shutdown</button>
                        <button className="stylized_button delete_house_button" onClick={this.deleteHouse}><i className="fa fa-close"></i> Delete house</button>
                    </div>
                    <div className="house_detail_block tabs">
                        <Tabs>
                            <TabList>
                                <Tab>Devices</Tab>
                                <Tab>Consumption events</Tab>
                            </TabList>

                            <TabPanel>
                                <div className="sort_data">
                                    <label>Sort by:</label>
                                    <select name="sortBy" id="sortBy" defaultValue={this.state.selectedOptionForDevices} onChange={this.onValueChangeDevices}>
                                        <option value="id">ID</option>
                                        <option value="nickName">Nickname</option>
                                        <option value="model">Model</option>
                                        <option value="powerStatus">Power status</option>
                                    </select>
                                    <select name="order" id="order" defaultValue={this.state.orderByForDevices} onChange={this.onValueChangeOrderDevices}>
                                        <option value=",asc">Ascending</option>
                                        <option value=",desc">Descending</option>
                                    </select>
                                </div>
                                {devices.length === 0 ?
                                    <>
                                        <p className="no_results">{":'("}</p>
                                        <p className="no_results">No devices found...</p>
                                    </> :
                                    <>
                                        <table>
                                            <thead>
                                                <tr>
                                                    <th>ID</th>
                                                    <th>Nickname</th>
                                                    <th>Model</th>
                                                    <th>Power Status</th>
                                                </tr>
                                            </thead>
                                            <tbody>
                                                {devices.map(device =>
                                                    <tr key={device.id} className="basic_row">
                                                        <td>{device.id}</td>
                                                        <td>{device.nickname}</td>
                                                        <td>{device.model}</td>
                                                        <td>{device.powerStatus}</td>
                                                    </tr>
                                                )}
                                            </tbody>
                                        </table>
                                        <div className="pagination_buttons" style={{ float: "left", fontFamily: "monospace", color: "#0275D8" }}>
                                            Page {currentPageDevices} of {totalPagesDevices}
                                        </div>
                                        <div className="pagination_buttons" style={{ float: "right" }}>
                                            <nav>
                                                <ul className="pagination">
                                                    <li className="page-item"><button className="page-link" disabled={currentPageDevices === 1 ? true : false} onClick={this.showFirstPageDevices}>{"<< First"}</button></li>
                                                    <li className="page-item"><button className="page-link" disabled={currentPageDevices === 1 ? true : false} onClick={this.showPrevPageDevices}>{"< Previous"}</button></li>
                                                    <li className="page-item"><button className="page-link" disabled={currentPageDevices === totalPagesDevices ? true : false} onClick={this.showNextPageDevices}>{"Next >"}</button></li>
                                                    <li className="page-item"><button className="page-link" disabled={currentPageDevices === totalPagesDevices ? true : false} onClick={this.showLastPageDevices}>{"Last >>"}</button></li>
                                                </ul>
                                            </nav>
                                        </div>
                                    </>
                                }
                            </TabPanel>
                            <TabPanel>
                                <div className="sort_data">
                                    <label>Sort by:</label>
                                    <select name="sortBy" id="sortBy" defaultValue={this.state.selectedOptionForConsumptionEvents} onChange={this.onValueChangeConsumptionEvents}>
                                        <option value="id">ID</option>
                                        <option value="timestamp">Timestamp</option>
                                        <option value="powerConsumption">Power Consumption</option>
                                        <option value="device">Device ID</option>
                                    </select>
                                    <select name="order" id="order" defaultValue={this.state.orderByForConsumptionEvents} onChange={this.onValueChangeOrderConsumptionEvents}>
                                        <option value=",asc">Ascending</option>
                                        <option value=",desc">Descending</option>
                                    </select>
                                </div>
                                {consumptionEvents.length === 0 ?
                                    <>
                                        <p className="no_results">{":'("}</p>
                                        <p className="no_results">No consumption events found...</p>
                                    </> :
                                    <>
                                        <table>
                                            <thead>
                                                <tr>
                                                    <th>ID</th>
                                                    <th>Timestamp</th>
                                                    <th>Power Consumption</th>
                                                    <th>Device ID</th>
                                                    <th>Device nickname</th>
                                                </tr>
                                            </thead>
                                            <tbody>
                                                {consumptionEvents.map(event =>
                                                    <tr key={event.id} className="basic_row">
                                                        <td>{event.id}</td>
                                                        <td>{event.timestamp}</td>
                                                        <td>{event.powerConsumption}</td>
                                                        <td>{event.device.id}</td>
                                                        <td>{event.device.nickname}</td>
                                                    </tr>
                                                )}
                                            </tbody>
                                        </table>
                                        <div className="pagination_buttons" style={{ float: "left", fontFamily: "monospace", color: "#0275D8" }}>
                                            Page {currentPageConsumptionEvents} of {totalPagesConsumptionEvents}
                                        </div>
                                        <div className="pagination_buttons" style={{ float: "right" }}>
                                            <nav>
                                                <ul className="pagination">
                                                    <li className="page-item"><button className="page-link" disabled={currentPageConsumptionEvents === 1 ? true : false} onClick={this.showFirstPageConsumptionEvents}>{"<< First"}</button></li>
                                                    <li className="page-item"><button className="page-link" disabled={currentPageConsumptionEvents === 1 ? true : false} onClick={this.showPrevPageConsumptionEvents}>{"< Previous"}</button></li>
                                                    <li className="page-item"><button className="page-link" disabled={currentPageConsumptionEvents === totalPagesConsumptionEvents ? true : false} onClick={this.showNextPageConsumptionEvents}>{"Next >"}</button></li>
                                                    <li className="page-item"><button className="page-link" disabled={currentPageConsumptionEvents === totalPagesConsumptionEvents ? true : false} onClick={this.showLastPageConsumptionEvents}>{"Last >>"}</button></li>
                                                </ul>
                                            </nav>
                                        </div>
                                    </>
                                }
                            </TabPanel>
                        </Tabs>
                    </div>
                </div>
            </div>
        );
    }
}
