import axios from 'axios';
import React from 'react';
import { Link } from 'react-router-dom';
import BackButton from '../BackButton';

interface IHousesProps {

}

interface IHousesState {
    houses: IHouse[];
    currentPage: number;
    recordsPerPage: number;
    totalPages: number;
    selectedOption: string;
    orderBy: string;
}

interface IHouse {
    id: string;
    city: string;
    country: string;
    name: string;
    street: string;
    zip: string;
    assignedDeviceCount: string;
}

export default class Houses extends React.Component<IHousesProps, IHousesState> {
    constructor(props: IHousesProps) {
        super(props);
        this.state = {
            houses: [],
            currentPage: 1,
            recordsPerPage: 20,
            totalPages: 0,
            selectedOption: "id",
            orderBy: ",asc"
        }

        this.onValueChange = this.onValueChange.bind(this);
        this.onValueChangeOrder = this.onValueChangeOrder.bind(this);
    }

    componentDidMount() {
        this.getHousesByPagination(this.state.currentPage);
    }

    getHousesByPagination(currentPage: number) {
        this.setState({ currentPage: currentPage });
        axios.get("/me/houses?page=" + (currentPage - 1) + "&size=" + this.state.recordsPerPage + "&sort=" + this.state.selectedOption + this.state.orderBy,
            {
                headers: {
                    "authorization": `${sessionStorage.getItem("token")}`
                }
            }
        ).then(response => {
            this.setState({
                houses: response.data
            })
        });

        axios.get("/me/houses/count",
            {
                headers: {
                    "authorization": `${sessionStorage.getItem("token")}`
                }
            }
        ).then(response => {
            var totalRecords = response.data;
            this.setState({
                totalPages: Math.ceil(totalRecords / this.state.recordsPerPage)
            })
        });
    }

    showFirstPage = () => {
        this.getHousesByPagination(1);
    }

    showPrevPage = () => {
        if (this.state.currentPage > 1) {
            this.getHousesByPagination(this.state.currentPage - 1);
        }
    }

    showNextPage = () => {
        this.getHousesByPagination(this.state.currentPage + 1);
    }

    showLastPage = () => {
        this.getHousesByPagination(this.state.totalPages);
    }

    onValueChange(event: any) {
        this.setState({
            selectedOption: event.target.value
        }, () => this.showFirstPage());
    }

    onValueChangeOrder(event: any) {
        this.setState({
            orderBy: event.target.value
        }, () => this.showFirstPage());
    }

    render() {
        const { houses, currentPage, totalPages } = this.state;
        return (
            <div className="layout">
                {BackButton("/")}
                <div>
                    <h2 className="heading">My houses</h2>
                    <div className="sort_data">
                        <label>Sort by:</label>
                        <select name="sortBy" id="sortBy" defaultValue={this.state.selectedOption} onChange={this.onValueChange}>
                            <option value="id">ID</option>
                            <option value="name">Name</option>
                            <option value="street">Street</option>
                            <option value="city">City</option>
                            <option value="zip">ZIP</option>
                            <option value="country">Country</option>
                        </select>
                        <select name="order" id="order" defaultValue={this.state.orderBy} onChange={this.onValueChangeOrder}>
                            <option value=",asc">Ascending</option>
                            <option value=",desc">Descending</option>
                        </select>
                    </div>
                    <div className="list">
                        <>
                            {houses.length === 0 ?
                                <>
                                    <p className="no_results">{":'("}</p>
                                    <p className="no_results">No houses found...</p>
                                </> :
                                <>
                                    <table>
                                        <thead>
                                            <tr>
                                                <th>ID</th>
                                                <th>City</th>
                                                <th>Country</th>
                                                <th>Name</th>
                                                <th>Street</th>
                                                <th>ZIP</th>
                                                <th>Assigned Device Count</th>
                                            </tr>
                                        </thead>
                                        <tbody>
                                            {houses.map(house =>
                                                <tr key={house.id} className="tbl_row" onClick={() => window.location.href = "houses/" + house.id + "?page=" + (this.state.currentPage - 1)}>
                                                    <td>{house.id}</td>
                                                    <td>{house.city}</td>
                                                    <td>{house.country}</td>
                                                    <td>{house.name}</td>
                                                    <td>{house.street}</td>
                                                    <td>{house.zip}</td>
                                                    <td>{house.assignedDeviceCount}</td>
                                                </tr>
                                            )}
                                        </tbody>
                                    </table>
                                    <div className="pagination_buttons" style={{ float: "left", fontFamily: "monospace", color: "#0275D8" }}>
                                        Page {currentPage} of {totalPages}
                                    </div>
                                    <div className="pagination_buttons" style={{ float: "right" }}>
                                        <nav>
                                            <ul className="pagination">
                                                <li className="page-item"><button className="page-link" disabled={currentPage === 1 ? true : false} onClick={this.showFirstPage}>{"<< First"}</button></li>
                                                <li className="page-item"><button className="page-link" disabled={currentPage === 1 ? true : false} onClick={this.showPrevPage}>{"< Previous"}</button></li>
                                                <li className="page-item"><button className="page-link" disabled={currentPage === totalPages ? true : false} onClick={this.showNextPage}>{"Next >"}</button></li>
                                                <li className="page-item"><button className="page-link" disabled={currentPage === totalPages ? true : false} onClick={this.showLastPage}>{"Last >>"}</button></li>
                                            </ul>
                                        </nav>
                                    </div>
                                </>
                            }
                        </>
                    </div>
                </div>
                <div className="add_button">
                    <Link to="/create-house">
                        <button className="stylized_button button_color"><i className="fa fa-plus"></i> Add new house</button>
                    </Link>
                </div>
            </div>
        );
    }
}
