import axios from "axios";
import React from "react";
import BackButton from "../BackButton";

interface IEditProfileProps {

}

interface IEditProfileState {
    changed: boolean;
    profile: IProfile;
}

interface IProfile {
    gender: string;
    firstName: string;
    lastName: string;
    phoneNumber: string;
    email: string;
}

export default class EditProfile extends React.Component<IEditProfileProps, IEditProfileState> {
    constructor(props: IEditProfileProps) {
        super(props);
        this.state = {
            changed: false,
            profile: {
                gender: "",
                firstName: "",
                lastName: "",
                phoneNumber: "",
                email: ""
            }
        }

        this.onValueChange = this.onValueChange.bind(this);
        this.handleSubmit = this.handleSubmit.bind(this);
    }

    componentDidMount() {
        this.getProfileData();
    }

    getProfileData() {
        axios.get("/users/current/profile",
            {
                headers: {
                    "authorization": `${sessionStorage.getItem("token")}`
                }
            }
        ).then(response => {
            this.setState({
                profile: response.data,
            });
        });
    }

    handleSubmit(event: any) {
        axios.put("/users/current/profile",
            JSON.stringify(
                {
                    gender: this.state.profile.gender,
                    firstName: event.target.firstName.value,
                    lastName: event.target.lastName.value,
                    phoneNumber: event.target.phoneNumber.value
                }
            ),
            {
                headers: {
                    "Content-Type": "application/json",
                    "authorization": `${sessionStorage.getItem("token")}`
                }
            }
        ).then(() => {
            this.setState({
                changed: true
            })
        });
    }

    onValueChange(event: any) {
        this.setState({
            profile: { ...this.state.profile, gender: event.target.value }
        });
    }

    render() {
        const { changed, profile } = this.state;
        if (changed) {
            window.location.href = "/pa165/profile";
        }
        return (
            <div>
                {BackButton("/profile")}
                <h2>Edit profile</h2>
                <form onSubmit={this.handleSubmit} className="custom_form">
                    <label>First name: </label>
                    <input type="text" name="firstName" defaultValue={profile.firstName} required /><br />
                    <label>Last name: </label>
                    <input type="text" name="lastName" defaultValue={profile.lastName} required /><br />
                    <label>Phone number: </label>
                    <input type="text" name="phoneNumber" defaultValue={profile.phoneNumber} required /><br />
                    <label>Gender: </label>
                    <select name="gender" id="gender" value={this.state.profile.gender} onChange={this.onValueChange}>
                        <option value="MALE">Male</option>
                        <option value="FEMALE">Female</option>
                        <option value="NON_BINARY">Non-binary</option>
                    </select><br />
                    <input className="stylized_button button_color" type="submit" value="Update" id="submit" />
                </form>
            </div>
        );
    }
}
