import axios from "axios";
import React from "react";
import { Redirect } from "react-router-dom";
import BackButton from "../BackButton";

interface IProfileProps {

}

interface IProfileState {
    exists: boolean;
    profile: IProfile;
}

interface IProfile {
    gender: string;
    firstName: string;
    lastName: string;
    phoneNumber: string;
    email: string;
}

export default class Profile extends React.Component<IProfileProps, IProfileState> {
    constructor(props: IProfileProps) {
        super(props);
        this.state = {
            exists: true,
            profile: {
                gender: "",
                firstName: "",
                lastName: "",
                phoneNumber: "",
                email: ""
            }
        }

        this.deleteAccount = this.deleteAccount.bind(this);
    }

    componentDidMount() {
        this.getProfileData();
    }

    getProfileData() {
        axios.get("/users/current/profile",
            {
                headers: {
                    "authorization": `${sessionStorage.getItem("token")}`
                }
            }
        ).then(response => {
            this.setState({
                profile: response.data
            })
        });
    }

    deleteAccount() {
        var result = window.confirm("Are you sure you want to delete your account?");
        if (result) {
            axios.delete("/users/current",
                {
                    headers: {
                        "authorization": `${sessionStorage.getItem("token")}`
                    }
                }
            ).then(() => {
                this.setState({
                    exists: false
                })
            });
            window.alert("Your account has been deleted!");
        }
    }

    render() {
        const { exists, profile } = this.state;
        if (!exists) {
            return <Redirect to="/login" />;
        }
        return (
            <>
                {BackButton("/")}
                <h2 className="heading">Profile</h2>
                <div className="profile">
                    <div className="profile_block">
                        <table>
                            <tbody>
                                <tr className="basic_row">
                                    <td>First name</td>
                                    <td>{profile.firstName}</td>
                                </tr>
                                <tr className="basic_row">
                                    <td>Last name</td>
                                    <td>{profile.lastName}</td>
                                </tr>
                                <tr className="basic_row">
                                    <td>Gender</td>
                                    <td>{profile.gender === "MALE" ? "Male" : (profile.gender === "FEMALE" ? "Female" : "Non-binary")}</td>
                                </tr>
                                <tr className="basic_row">
                                    <td>E-mail</td>
                                    <td>{profile.email}</td>
                                </tr>
                                <tr className="basic_row">
                                    <td>Phone number</td>
                                    <td>{profile.phoneNumber}</td>
                                </tr>
                                <tr className="basic_row">
                                    <td>Role</td>
                                    <td>{sessionStorage.getItem("authority") === "DISTRIBUTOR" ? "Distributor" : "User"}</td>
                                </tr>
                            </tbody>
                        </table>
                    </div>
                    <div className="profile_block profile_buttons">
                        <button className="stylized_button profile_button button_color" onClick={() => window.location.href = "profile/edit"}><i className="fa fa-edit"></i> Edit profile</button>
                        <button className="stylized_button profile_button button_color" onClick={() => window.location.href = "profile/change-password"}>Change password</button>
                        <button className="stylized_button profile_button delete_acc_button" onClick={this.deleteAccount}><i className="fa fa-remove"></i> Delete account</button>
                    </div>
                </div>
            </>
        );
    }
}
