package cz.muni.fi.pa165.project.rest.controller;

import cz.muni.fi.pa165.project.dto.DistributorConsumptionEventListDTO;
import cz.muni.fi.pa165.project.facade.ConsumptionEventFacade;
import cz.muni.fi.pa165.project.rest.ApiUris;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Pageable;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

@RestController
@RequestMapping(ApiUris.URI_CONSUMPTION_EVENTS)
@Tag(name = "Consumption events", description = "Management of consumption events")
public class ConsumptionEventController {
    private final ConsumptionEventFacade consumptionEventFacade;

    @Autowired
    public ConsumptionEventController(ConsumptionEventFacade consumptionEventFacade) {
        this.consumptionEventFacade = consumptionEventFacade;
    }

    @Operation(description = "Returns all consumption events")
    @GetMapping
    public final ResponseEntity<List<DistributorConsumptionEventListDTO>> getAll(Pageable pageable) {
        return ResponseEntity.ok().body(consumptionEventFacade.getAll(pageable).getContent());
    }

    @Operation(description = "Returns count of all consumption events")
    @GetMapping("/count")
    public final ResponseEntity<Long> getAllCount(Pageable pageable) {
        return ResponseEntity.ok().body(consumptionEventFacade.getAll(pageable).getTotalElements());
    }
}
