package cz.muni.fi.pa165.project.rest.controller;

import cz.muni.fi.pa165.project.dto.DeviceCreateDTO;
import cz.muni.fi.pa165.project.dto.DeviceListDTO;
import cz.muni.fi.pa165.project.facade.DeviceFacade;
import cz.muni.fi.pa165.project.rest.ApiUris;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Pageable;
import org.springframework.http.ResponseEntity;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

@RestController
@RequestMapping(ApiUris.URI_DEVICES_USER)
@Tag(name = "Devices", description = "Management of devices")
public class UserDeviceController {
    private static final Logger LOGGER = LoggerFactory.getLogger(UserDeviceController.class);
    private final DeviceFacade deviceFacade;

    @Autowired
    public UserDeviceController(DeviceFacade deviceFacade) {
        this.deviceFacade = deviceFacade;
    }

    @Operation(description = "Returns all devices of the current user")
    @GetMapping
    public final ResponseEntity<List<DeviceListDTO>> getAll(Pageable pageable) {
        var loggedInUsername = ((UserDetails) SecurityContextHolder.getContext().getAuthentication().getPrincipal()).getUsername();
        try {
            return ResponseEntity.ok().body(deviceFacade.getDevicesByUser(loggedInUsername, pageable).getContent());
        } catch (Exception e) {
            LOGGER.error("Could not retrieve devices for current user.", e);
            return ResponseEntity.notFound().build();
        }
    }

    @Operation(description = "Returns count of all devices of the current user")
    @GetMapping("/count")
    public final ResponseEntity<Long> getAllCount(Pageable pageable) {
        var loggedInUsername = ((UserDetails) SecurityContextHolder.getContext().getAuthentication().getPrincipal()).getUsername();
        try {
            return ResponseEntity.ok().body(deviceFacade.getDevicesByUser(loggedInUsername, pageable).getTotalElements());
        } catch (Exception e) {
            LOGGER.error("Could not retrieve the count of devices for current user.", e);
            return ResponseEntity.notFound().build();
        }
    }

    @Operation(description = "Creates a device for the current user")
    @PostMapping
    public final ResponseEntity<Long> create(@RequestBody DeviceCreateDTO device) {
        var loggedInUsername = ((UserDetails) SecurityContextHolder.getContext().getAuthentication().getPrincipal()).getUsername();
        try {
            return ResponseEntity.ok().body(deviceFacade.createDevice(loggedInUsername, device));
        } catch (Exception e) {
            LOGGER.error("Could not create device for current user.", e);
            return ResponseEntity.notFound().build();
        }
    }
}
