package cz.muni.fi.pa165.project.rest.controller;

import cz.muni.fi.pa165.project.dto.HouseCreateDTO;
import cz.muni.fi.pa165.project.dto.HouseListDTO;
import cz.muni.fi.pa165.project.facade.HouseFacade;
import cz.muni.fi.pa165.project.rest.ApiUris;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Pageable;
import org.springframework.http.ResponseEntity;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

@RestController
@RequestMapping(ApiUris.URI_HOUSES_USER)
@Tag(name = "Houses", description = "Management of houses")
public class UserHouseController {
    private static final Logger LOGGER = LoggerFactory.getLogger(UserHouseController.class);
    private final HouseFacade houseFacade;

    @Autowired
    public UserHouseController(HouseFacade houseFacade) {
        this.houseFacade = houseFacade;
    }

    @Operation(description = "Returns all houses of the current user")
    @GetMapping
    public final ResponseEntity<List<HouseListDTO>> getAll(Pageable pageable) {
        var loggedInUsername = ((UserDetails) SecurityContextHolder.getContext().getAuthentication().getPrincipal()).getUsername();
        try {
            return ResponseEntity.ok().body(houseFacade.getHousesByUser(loggedInUsername, pageable).getContent());
        } catch (Exception e) {
            LOGGER.error("Could not retrieve houses for current user.", e);
            return ResponseEntity.notFound().build();
        }
    }

    @Operation(description = "Returns count of all houses of the current user")
    @GetMapping("/count")
    public final ResponseEntity<Long> getAllCount(Pageable pageable) {
        var loggedInUsername = ((UserDetails) SecurityContextHolder.getContext().getAuthentication().getPrincipal()).getUsername();
        try {
            return ResponseEntity.ok().body(houseFacade.getHousesByUser(loggedInUsername, pageable).getTotalElements());
        } catch (Exception e) {
            LOGGER.error("Could not retrieve the count of houses for current user.", e);
            return ResponseEntity.notFound().build();
        }
    }

    @Operation(description = "Creates a new house for the current user")
    @PostMapping
    public final ResponseEntity<Long> create(@RequestBody HouseCreateDTO house) {
        var loggedInUsername = ((UserDetails) SecurityContextHolder.getContext().getAuthentication().getPrincipal()).getUsername();
        try {
            return ResponseEntity.ok().body(houseFacade.createHouse(loggedInUsername, house));
        } catch (Exception e) {
            LOGGER.error("Could not create house for current user.", e);
            return ResponseEntity.notFound().build();
        }
    }
}
