package cz.muni.fi.pa165.project;

import org.springframework.beans.BeansException;
import org.springframework.beans.factory.config.BeanFactoryPostProcessor;
import org.springframework.beans.factory.config.ConfigurableListableBeanFactory;
import org.springframework.stereotype.Component;

import java.util.Arrays;

/**
 * Enables lazy initialization of all beans during tests to avoid
 * creating controllers we are currently not trying to test.
 *
 * @author Timotej Šulík | 493364
 */
@Component
public class TestLazyBeanInitConfiguration implements BeanFactoryPostProcessor {

    @Override
    public void postProcessBeanFactory(ConfigurableListableBeanFactory beanFactory) throws BeansException {
        Arrays.stream(beanFactory.getBeanDefinitionNames())
                .map(beanFactory::getBeanDefinition)
                .forEach(beanDefinition -> beanDefinition.setLazyInit(true));
    }
}