package cz.muni.fi.pa165.project.rest.controller;

import cz.muni.fi.pa165.project.dto.DistributorConsumptionEventListDTO;
import cz.muni.fi.pa165.project.facade.ConsumptionEventFacade;
import org.junit.jupiter.api.Test;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.PageRequest;
import org.springframework.http.MediaType;
import org.springframework.security.test.context.support.WithMockUser;

import java.time.LocalDateTime;
import java.util.Arrays;
import java.util.List;

import static org.mockito.Mockito.when;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.content;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

public class ConsumptionEventControllerTest extends BaseControllerTest<ConsumptionEventFacade, ConsumptionEventController> {

    @WithMockUser(username = "user@mail.com", roles = {"DISTRIBUTOR"})
    @Test
    public void getConsumptionEvents() throws Exception {
        var device = createConsumptionEvents();

        when(facade.getAll(PageRequest.of(0, 2))).thenReturn(new PageImpl<>(device));

        mockMvc.perform(get("/consumption_events?page=0&size=2"))
                .andExpect(status().isOk())
                .andExpect(content().contentTypeCompatibleWith(MediaType.APPLICATION_JSON_VALUE))
                .andExpect(content().string(JsonStringMatcher.of(device)));
    }

    private List<DistributorConsumptionEventListDTO> createConsumptionEvents() {
        var eventOne = new DistributorConsumptionEventListDTO(486, LocalDateTime.of(2021, 12, 12, 1, 1), 2.5, "deva");
        var eventTwo = new DistributorConsumptionEventListDTO(10, LocalDateTime.of(2022, 12, 12, 1, 2), 20, "devb");
        return Arrays.asList(eventOne, eventTwo);
    }
}
