package cz.muni.fi.pa165.project.rest.controller;

import cz.muni.fi.pa165.project.dto.DeviceAssignHouseDTO;
import cz.muni.fi.pa165.project.dto.DeviceDetailConsumptionEventDTO;
import cz.muni.fi.pa165.project.dto.DeviceDetailDTO;
import cz.muni.fi.pa165.project.dto.DeviceDetailHouseDTO;
import cz.muni.fi.pa165.project.dto.HouseNameDTO;
import cz.muni.fi.pa165.project.enums.PowerStatusDto;
import cz.muni.fi.pa165.project.facade.DeviceFacade;
import org.junit.jupiter.api.Disabled;
import org.junit.jupiter.api.Test;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.PageRequest;
import org.springframework.http.MediaType;
import org.springframework.security.test.context.support.WithMockUser;

import java.time.LocalDateTime;
import java.util.Arrays;
import java.util.List;

import static org.mockito.Mockito.times;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.verifyNoMoreInteractions;
import static org.mockito.Mockito.when;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultHandlers.print;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.content;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

public class DeviceControllerTest extends BaseControllerTest<DeviceFacade, DeviceController> {

    @WithMockUser(username = "user@mail.com", roles = {"APP_USER"})
    @Test
    public void getDeviceDetail() throws Exception {
        DeviceDetailHouseDTO house = new DeviceDetailHouseDTO(69, "house");
        DeviceDetailDTO device = new DeviceDetailDTO(16, "nickname", "model", PowerStatusDto.ON, house);

        when(facade.getDevice("user@mail.com", 564)).thenReturn(device);

        mockMvc.perform(get("/devices/564"))
                .andExpect(status().isOk())
                .andExpect(content().contentTypeCompatibleWith(MediaType.APPLICATION_JSON_VALUE))
                .andExpect(content().string(JsonStringMatcher.of(device)));
    }

    @WithMockUser(username = "user@mail.com", roles = {"APP_USER"})
    @Test
    public void deleteDevice() throws Exception {
        var request = delete("/devices/152");
        mockMvc.perform(request)
                .andExpect(status().isOk());

        verify(facade, times(1)).deleteDevice("user@mail.com", 152);
        verifyNoMoreInteractions(facade);
    }

    @WithMockUser(username = "user@mail.com", roles = {"APP_USER"})
    @Test
    @Disabled("Removed assign date from device facade, possible errors")
    public void assignToHouse() throws Exception {
        var dto = new DeviceAssignHouseDTO(LocalDateTime.of(2045, 1, 2, 4, 5));

        var request = post("/devices/13/houses/1022")
                .contentType(MediaType.APPLICATION_JSON)
                .content(TestJsonSerializer.toJson(dto));
        mockMvc.perform(request)
                .andExpect(status().isOk());

        verify(facade, times(1)).assignDeviceToHouse("user@mail.com", 13, 1022);
    }

    @WithMockUser(username = "user@mail.com", roles = {"APP_USER"})
    @Test
    public void unAssignFromHouse() throws Exception {
        var request = delete("/devices/14/houses/1025");
        mockMvc.perform(request)
                .andExpect(status().isOk());

        verify(facade, times(1)).unAssignDeviceFromHouse("user@mail.com", 14, 1025);
    }

    @WithMockUser(username = "user@mail.com", roles = {"APP_USER"})
    @Test
    public void getConsumptionEvents() throws Exception {
        var devices = createConsumptionEvents();

        when(facade.getConsumptionEventsByDevice("user@mail.com", 10000, PageRequest.of(0, 2)))
                .thenReturn(new PageImpl<>(devices));

        mockMvc.perform(get("/devices/10000/consumption_events?page=0&size=2"))
                .andDo(print())
                .andExpect(status().isOk())
                .andExpect(content().contentTypeCompatibleWith(MediaType.APPLICATION_JSON_VALUE))
                .andExpect(content().string(JsonStringMatcher.of(devices)));
    }

    private List<DeviceDetailConsumptionEventDTO> createConsumptionEvents() {
        var deviceOne = new HouseNameDTO(123, "deva");
        var eventOne = new DeviceDetailConsumptionEventDTO(486, deviceOne, LocalDateTime.of(2021, 12, 12, 1, 1), 2.5);
        var deviceTwo = new HouseNameDTO(1489, "devb");
        var eventTwo = new DeviceDetailConsumptionEventDTO(10, deviceTwo, LocalDateTime.of(2022, 12, 12, 1, 2), 20);
        return Arrays.asList(eventOne, eventTwo);
    }
}
