package cz.muni.fi.pa165.project.rest.controller;

import cz.muni.fi.pa165.project.dto.UserChangePasswordDTO;
import cz.muni.fi.pa165.project.dto.UserProfileDTO;
import cz.muni.fi.pa165.project.dto.UserUpdateDTO;
import cz.muni.fi.pa165.project.enums.GenderDto;
import cz.muni.fi.pa165.project.facade.UserFacade;
import org.junit.jupiter.api.Test;
import org.springframework.http.MediaType;
import org.springframework.security.test.context.support.WithMockUser;

import static org.mockito.Mockito.times;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.verifyNoMoreInteractions;
import static org.mockito.Mockito.when;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.delete;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.put;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.content;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

public class UserControllerTest extends BaseControllerTest<UserFacade, UserController> {

    @WithMockUser(username = "user@mail.com", roles = {"APP_USER"})
    @Test
    public void getCurrentUser() throws Exception {
        var user = new UserProfileDTO(GenderDto.FEMALE, "jozo", "mrkva", "+420698789789", "jozo@mrkva.cz");

        when(facade.getUserByUserName("user@mail.com")).thenReturn(user);

        mockMvc.perform(get("/users/current/profile"))
                .andExpect(status().isOk())
                .andExpect(content().contentTypeCompatibleWith(MediaType.APPLICATION_JSON_VALUE))
                .andExpect(content().string(JsonStringMatcher.of(user)));
    }

    @WithMockUser(username = "user@mail.com", roles = {"APP_USER"})
    @Test
    public void deleteUser() throws Exception {
        var request = delete("/users/current");
        mockMvc.perform(request)
                .andExpect(status().isOk());

        verify(facade, times(1)).deleteUser("user@mail.com");
        verifyNoMoreInteractions(facade);
    }

    @WithMockUser(username = "user@mail.com", roles = {"APP_USER"})
    @Test
    public void updateUserProfile() throws Exception {
        var device = new UserUpdateDTO(GenderDto.NON_BINARY, "Stevo", "Skrucany", "+420698789456");

        var request = put("/users/current/profile")
                .contentType(MediaType.APPLICATION_JSON)
                .content(TestJsonSerializer.toJson(device));
        mockMvc.perform(request)
                .andExpect(status().isOk());

        verify(facade, times(1)).updateUser("user@mail.com", device);
        verifyNoMoreInteractions(facade);
    }

    @WithMockUser(username = "user@mail.com", roles = {"APP_USER"})
    @Test
    public void changePassword() throws Exception {
        var dto = new UserChangePasswordDTO("a", "b", "b");

        var request = put("/users/current/password")
                .contentType(MediaType.APPLICATION_JSON)
                .content(TestJsonSerializer.toJson(dto));
        mockMvc.perform(request)
                .andExpect(status().isOk());

        verify(facade, times(1)).changePassword("user@mail.com", dto);
        verifyNoMoreInteractions(facade);
    }
}
