package cz.muni.fi.pa165.project.rest.controller;

import cz.muni.fi.pa165.project.dto.DeviceCreateDTO;
import cz.muni.fi.pa165.project.dto.DeviceListDTO;
import cz.muni.fi.pa165.project.dto.DeviceListHouseDTO;
import cz.muni.fi.pa165.project.enums.PowerStatusDto;
import cz.muni.fi.pa165.project.facade.DeviceFacade;
import org.junit.jupiter.api.Test;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.PageRequest;
import org.springframework.http.MediaType;
import org.springframework.security.test.context.support.WithMockUser;

import java.util.Arrays;
import java.util.List;

import static org.mockito.Mockito.when;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.post;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.content;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

public class UserDeviceControllerTest extends BaseControllerTest<DeviceFacade, UserDeviceController> {

    @WithMockUser(username = "user@mail.com", roles = {"APP_USER"})
    @Test
    public void getDevicesByUser() throws Exception {
        var devices = createDevices();

        when(facade.getDevicesByUser("user@mail.com", PageRequest.of(0, 2))).thenReturn(new PageImpl<>(devices));

        mockMvc.perform(get("/me/devices?page=0&size=2"))
                .andExpect(status().isOk())
                .andExpect(content().contentTypeCompatibleWith(MediaType.APPLICATION_JSON_VALUE))
                .andExpect(content().string(JsonStringMatcher.of(devices)));
    }

    @WithMockUser(username = "user@mail.com", roles = {"APP_USER"})
    @Test
    public void createDevice() throws Exception {
        var device = new DeviceCreateDTO("nickname", "model");

        when(facade.createDevice("user@mail.com", device)).thenReturn(10L);

        var request = post("/me/devices")
                .contentType(MediaType.APPLICATION_JSON)
                .content(TestJsonSerializer.toJson(device));
        mockMvc.perform(request)
                .andExpect(status().isOk())
                .andExpect(content().contentTypeCompatibleWith(MediaType.APPLICATION_JSON_VALUE))
                .andExpect(content().string("10"));
    }

    private List<DeviceListDTO> createDevices() {
        DeviceListHouseDTO houseOne = new DeviceListHouseDTO(1, "house");
        DeviceListDTO deviceOne = new DeviceListDTO(0, "a", "b", PowerStatusDto.ON, houseOne);

        DeviceListHouseDTO houseTwo = new DeviceListHouseDTO(3, "mouse");
        DeviceListDTO deviceTwo = new DeviceListDTO(2, "c", "d", PowerStatusDto.OFF, houseTwo);

        return Arrays.asList(deviceOne, deviceTwo);
    }
}
