package cz.muni.fi.pa165.project.rest.controller;

import cz.muni.fi.pa165.project.dto.HouseCreateDTO;
import cz.muni.fi.pa165.project.dto.HouseListDTO;
import cz.muni.fi.pa165.project.facade.HouseFacade;
import org.junit.jupiter.api.Test;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.PageRequest;
import org.springframework.http.MediaType;
import org.springframework.security.test.context.support.WithMockUser;

import java.util.Arrays;
import java.util.List;

import static org.mockito.Mockito.when;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.post;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.content;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

public class UserHouseControllerTest extends BaseControllerTest<HouseFacade, UserHouseController> {

    @WithMockUser(username = "user@mail.com", roles = {"APP_USER"})
    @Test
    public void getDevicesByUser() throws Exception {
        var devices = createHouses();

        when(facade.getHousesByUser("user@mail.com", PageRequest.of(0, 2))).thenReturn(new PageImpl<>(devices));

        mockMvc.perform(get("/me/houses?page=0&size=2"))
                .andExpect(status().isOk())
                .andExpect(content().contentTypeCompatibleWith(MediaType.APPLICATION_JSON_VALUE))
                .andExpect(content().string(JsonStringMatcher.of(devices)));
    }

    @WithMockUser(username = "user@mail.com", roles = {"APP_USER"})
    @Test
    public void createHouse() throws Exception {
        var house = new HouseCreateDTO("name", "street", "city", "61200", "CZ");

        when(facade.createHouse("user@mail.com", house)).thenReturn(20L);

        var request = post("/me/houses")
                .contentType(MediaType.APPLICATION_JSON)
                .content(TestJsonSerializer.toJson(house));
        mockMvc.perform(request)
                .andExpect(status().isOk())
                .andExpect(content().contentTypeCompatibleWith(MediaType.APPLICATION_JSON_VALUE))
                .andExpect(content().string("20"));
    }

    private List<HouseListDTO> createHouses() {
        HouseListDTO deviceOne = new HouseListDTO(1, "name", "street", "city", "61200", "CZ", 15);
        HouseListDTO deviceTwo = new HouseListDTO(1, "name2", "street2", "city2", "61300", "SK", 20);

        return Arrays.asList(deviceOne, deviceTwo);
    }
}
