package cz.muni.fi.pa165.project.config;

import cz.muni.fi.pa165.project.dto.DeviceDetailConsumptionEventDTO;
import cz.muni.fi.pa165.project.dto.HouseNameDTO;
import cz.muni.fi.pa165.project.entity.ConsumptionEvent;
import cz.muni.fi.pa165.project.entity.House;
import org.mapstruct.Mapper;
import org.springframework.data.domain.Page;

/**
 * @author Daniela Hostačná | 485432
 */
@Mapper(componentModel = "spring")
public interface DeviceDetailConsumptionEventMapper {
    default DeviceDetailConsumptionEventDTO map(ConsumptionEvent source) {
        if (source == null) {
            return null;
        }

        var assignedHouse = source.getDevice().getHouseDevices().stream()
                .filter(houseDevice -> houseDevice.getValidFrom().isBefore(source.getTimestamp()))
                .filter(houseDevice -> houseDevice.getValidTo() == null || houseDevice.getValidTo().isAfter(source.getTimestamp()))
                .findFirst().orElseThrow()
                .getHouse();

        return new DeviceDetailConsumptionEventDTO(
                source.getId() == null ? 0L : source.getId(),
                map(assignedHouse),
                source.getTimestamp(),
                source.getPowerConsumption()
        );
    }

    HouseNameDTO map(House source);

    default Page<DeviceDetailConsumptionEventDTO> map(Page<ConsumptionEvent> source) {
        return source.map(this::map);
    }
}
