package cz.muni.fi.pa165.project.config;

import cz.muni.fi.pa165.project.dto.DeviceListDTO;
import cz.muni.fi.pa165.project.dto.DeviceListHouseDTO;
import cz.muni.fi.pa165.project.entity.Device;
import cz.muni.fi.pa165.project.entity.House;
import org.mapstruct.Mapper;
import org.mapstruct.factory.Mappers;
import org.springframework.data.domain.Page;

/**
 * @author Daniela Hostačná | 485432
 */
@Mapper(componentModel = "spring")
public interface DeviceListMapper {
    default DeviceListDTO map(Device source) {
        if (source == null) {
            return null;
        }

        var activeHouseDevice = source.getHouseDevices().stream()
                .filter(houseDevice -> houseDevice.getValidTo() == null)
                .findFirst()
                .orElse(null);

        PowerStatusEnumMapper powerStatusEnumMapper = Mappers.getMapper(PowerStatusEnumMapper.class);

        return new DeviceListDTO(
                source.getId() == null ? 0L : source.getId(),
                source.getNickName(),
                source.getModel(),
                powerStatusEnumMapper.map(source.getPowerStatus()),
                activeHouseDevice == null ? null : map(activeHouseDevice.getHouse())
        );
    }

    DeviceListHouseDTO map(House source);

    default Page<DeviceListDTO> map(Page<Device> source) {
        if (source == null) {
            return null;
        }

        return source.map(this::map);
    }
}
