package cz.muni.fi.pa165.project.service;

import cz.muni.fi.pa165.project.entity.AppUser;
import cz.muni.fi.pa165.project.entity.BaseEntity;
import cz.muni.fi.pa165.project.entity.Device;
import cz.muni.fi.pa165.project.enums.PowerStatus;
import cz.muni.fi.pa165.project.repository.DeviceRepository;
import cz.muni.fi.pa165.project.repository.HouseDeviceRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.stream.Collectors;

/**
 * @author Daniela Hostačná | 485432
 */
@Service
public class DeviceServiceImpl implements DeviceService {
    private final ConsumptionEventService consumptionEventService;

    private final DeviceRepository deviceRepository;
    private final HouseDeviceRepository houseDeviceRepository;

    @Autowired
    public DeviceServiceImpl(DeviceRepository deviceRepository, HouseDeviceRepository houseDeviceRepository, ConsumptionEventService consumptionEventService) {
        this.deviceRepository = deviceRepository;
        this.houseDeviceRepository = houseDeviceRepository;
        this.consumptionEventService = consumptionEventService;
    }

    @Override
    public void deleteAllForUser(AppUser user) {
        var deviceIds = user
                .getDevices()
                .stream()
                .map(BaseEntity::getId)
                .collect(Collectors.toList());
        consumptionEventService.deleteAllByDeviceIds(deviceIds);
        houseDeviceRepository.deleteAllById(user
                .getDevices()
                .stream()
                .map(device -> device
                        .getHouseDevices()
                        .stream()
                        .map(BaseEntity::getId)
                        .collect(Collectors.toList()))
                .flatMap(List::stream)
                .collect(Collectors.toList()));
        deviceRepository.deleteAllById(deviceIds);
    }

    @Override
    public void shutDownAllDevicesByIds(List<Long> ids) {
        var devices = deviceRepository.findAllById(ids);
        devices.forEach(device -> device.setPowerStatus(PowerStatus.OFF));
    }

    @Override
    public Page<Device> findDevicesByIds(List<Long> ids, Pageable page) {
        return deviceRepository.findAllByIds(ids, page);
    }

    @Override
    public Page<Device> findDevicesForUser(AppUser user, Pageable page) {
        return deviceRepository.findAllByUser(user, page);
    }

    @Override
    public Device findById(long deviceId) {
        return deviceRepository.findById(deviceId).orElse(null);
    }

    @Override
    public Long create(Device device) {
        device.setPowerStatus(PowerStatus.OFF);
        return deviceRepository.save(device).getId();
    }

    @Override
    public void update(Device device) {
        var storedEntity = deviceRepository.findById(device.getId()).orElseThrow();
        storedEntity.setNickName(device.getNickName());
        storedEntity.setModel(device.getModel());
        storedEntity.setPowerStatus(device.getPowerStatus());
    }

    @Override
    public void delete(Device device) {
        consumptionEventService.deleteAllByDeviceIds(List.of(device.getId()));
        houseDeviceRepository.deleteAllById(device
                .getHouseDevices()
                .stream()
                .map(BaseEntity::getId)
                .collect(Collectors.toList()));
        deviceRepository.delete(device);
    }
}
