package cz.muni.fi.pa165.project.service;

import cz.muni.fi.pa165.project.entity.Device;
import cz.muni.fi.pa165.project.entity.House;
import cz.muni.fi.pa165.project.entity.HouseDevice;
import cz.muni.fi.pa165.project.repository.HouseDeviceRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.Clock;
import java.time.LocalDateTime;

/**
 * @author Daniela Hostačná | 485432
 */
@Service
public class HouseDeviceServiceImpl implements HouseDeviceService {
    private final HouseDeviceRepository houseDeviceRepository;
    private final Clock clock;


    @Autowired
    public HouseDeviceServiceImpl(HouseDeviceRepository houseDeviceRepository, Clock clock) {
        this.houseDeviceRepository = houseDeviceRepository;
        this.clock = clock;
    }

    @Override
    public long create(HouseDevice assignment) {
        return houseDeviceRepository.save(assignment).getId();
    }

    @Override
    public void closeAssignmentFor(Device device, House house) {
        var houseDevices = houseDeviceRepository.findOpenByDeviceAndHouse(device, house);
        if (houseDevices.isEmpty()) {
            throw new IllegalArgumentException("No active assignment to close");
        }
        if (houseDevices.size() > 1) {
            throw new IllegalArgumentException("More active assignments.");
        }
        houseDevices.get(0).setValidTo(LocalDateTime.now(clock));
    }
}
