package cz.muni.fi.pa165.project.service;

import cz.muni.fi.pa165.project.entity.AppUser;
import cz.muni.fi.pa165.project.entity.BaseEntity;
import cz.muni.fi.pa165.project.entity.House;
import cz.muni.fi.pa165.project.repository.HouseDeviceRepository;
import cz.muni.fi.pa165.project.repository.HouseRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.stream.Collectors;

/**
 * @author Daniela Hostačná | 485432
 */
@Service
public class HouseServiceImpl implements HouseService {
    private final HouseDeviceRepository houseDeviceRepository;
    private final HouseRepository houseRepository;

    @Autowired
    public HouseServiceImpl(HouseDeviceRepository houseDeviceRepository, HouseRepository houseRepository) {
        this.houseDeviceRepository = houseDeviceRepository;
        this.houseRepository = houseRepository;
    }

    @Override
    public House findById(long id) {
        return houseRepository.findById(id).orElse(null);
    }

    @Override
    public long create(House house) {
        return houseRepository.save(house).getId();
    }

    @Override
    public void deleteAllForUser(AppUser user) {
        houseDeviceRepository.deleteAllById(user
                .getHouses()
                .stream()
                .map(house -> house
                        .getHouseDevices()
                        .stream()
                        .map(BaseEntity::getId)
                        .collect(Collectors.toList()))
                .flatMap(List::stream)
                .collect(Collectors.toList()));
        houseRepository.deleteAllById(user
                .getHouses()
                .stream()
                .map(BaseEntity::getId)
                .collect(Collectors.toList()));
    }

    @Override
    public Page<House> findHousesForUser(AppUser user, Pageable page) {
        return houseRepository.findAllByUser(user, page);

    }

    @Override
    public void delete(House house) {
        houseDeviceRepository.deleteAllById(house
                .getHouseDevices()
                .stream()
                .map(BaseEntity::getId)
                .collect(Collectors.toList()));
        houseRepository.delete(house);
    }

    @Override
    public void delete(long houseId) {
        houseRepository.deleteById(houseId);
    }

    @Override
    public void update(House house) {
        var storedEntity = houseRepository.findById(house.getId()).orElseThrow();
        storedEntity.setName(house.getName());
        storedEntity.setStreet(house.getStreet());
        storedEntity.setCity(house.getCity());
        storedEntity.setZip(house.getZip());
        storedEntity.setCountry(house.getCountry());
    }
}
