package cz.muni.fi.pa165.project.config;

import cz.muni.fi.pa165.project.BaseServiceLayerTest;
import cz.muni.fi.pa165.project.dto.DeviceDetailConsumptionEventDTO;
import cz.muni.fi.pa165.project.dto.HouseNameDTO;
import cz.muni.fi.pa165.project.entity.AppUser;
import cz.muni.fi.pa165.project.entity.ConsumptionEvent;
import cz.muni.fi.pa165.project.entity.Device;
import cz.muni.fi.pa165.project.entity.House;
import cz.muni.fi.pa165.project.entity.HouseDevice;
import cz.muni.fi.pa165.project.enums.Gender;
import cz.muni.fi.pa165.project.enums.PowerStatus;
import cz.muni.fi.pa165.project.enums.UserRole;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;

import java.time.LocalDateTime;
import java.util.Set;

import static org.assertj.core.api.Assertions.assertThat;

/**
 * @author Daniela Hostačná | 485432
 */
class DeviceDetailConsumptionEventMapperTest extends BaseServiceLayerTest {
    @Autowired
    private DeviceDetailConsumptionEventMapper mapper;

    @Test
    void mapEntityToDto() {
        AppUser user = new AppUser("email", "hash", "firstName", "lastName", "123456789", Gender.FEMALE, UserRole.APP_USER);
        House house = new House(user, "FI MUNI", "Brno", "CZ", "Botanicka 68A", "60200");
        Device device = new Device(user, "Nice Device", "Model 123", PowerStatus.ON);
        HouseDevice houseDevice = new HouseDevice(house, device, LocalDateTime.now().minusDays(1), null);
        ConsumptionEvent consumptionEvent = new ConsumptionEvent(device, LocalDateTime.now(), 100);
        ConsumptionEvent notValidConsumptionEvent = new ConsumptionEvent(device, LocalDateTime.now().minusDays(10), 1);
        house.setHouseDevices(Set.of(houseDevice));
        device.setHouseDevices(Set.of(houseDevice));
        device.setConsumptionEvents(Set.of(notValidConsumptionEvent, consumptionEvent));
        device.setConsumptionEvents(Set.of(consumptionEvent));

        DeviceDetailConsumptionEventDTO dto = mapper.map(consumptionEvent);

        assertThat(dto).isNotNull();
        assertThat(dto.getId()).isEqualTo(0L);
        assertThat(dto.getHouse()).isEqualTo(new HouseNameDTO(0L, house.getName()));
        assertThat(dto.getTimestamp()).isEqualTo(consumptionEvent.getTimestamp());
        assertThat(dto.getPowerConsumption()).isEqualTo(consumptionEvent.getPowerConsumption());
    }

}