package cz.muni.fi.pa165.project.config;

import cz.muni.fi.pa165.project.BaseServiceLayerTest;
import cz.muni.fi.pa165.project.dto.DeviceDetailDTO;
import cz.muni.fi.pa165.project.dto.DeviceDetailHouseDTO;
import cz.muni.fi.pa165.project.entity.AppUser;
import cz.muni.fi.pa165.project.entity.Device;
import cz.muni.fi.pa165.project.entity.House;
import cz.muni.fi.pa165.project.entity.HouseDevice;
import cz.muni.fi.pa165.project.enums.Gender;
import cz.muni.fi.pa165.project.enums.PowerStatus;
import cz.muni.fi.pa165.project.enums.PowerStatusDto;
import cz.muni.fi.pa165.project.enums.UserRole;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;

import java.time.LocalDateTime;
import java.util.Set;

import static org.assertj.core.api.Assertions.assertThat;

/**
 * @author Daniela Hostačná | 485432
 */
class DeviceDetailMapperTest extends BaseServiceLayerTest {
    @Autowired
    private DeviceDetailMapper mapper;

    @Test
    void mapEntityToDto() {
        AppUser user = new AppUser("email", "hash", "firstName", "lastName", "123456789", Gender.FEMALE, UserRole.APP_USER);
        House house = new House(user, "FI MUNI", "Botanicka 68A", "Brno", "60200", "CZ");
        House previousHouse = new House(user, "FIT VUT", "Sever Brna", "Brno", "60200", "CZ");
        Device device = new Device(user, "Nice Device", "Model 123", PowerStatus.ON);
        HouseDevice houseDevice = new HouseDevice(house, device, LocalDateTime.now().minusDays(1), null);
        HouseDevice endedHouseDevice = new HouseDevice(previousHouse, device, LocalDateTime.now().minusDays(10), LocalDateTime.now().minusDays(1).minusMinutes(1));
        house.setHouseDevices(Set.of(houseDevice));
        previousHouse.setHouseDevices(Set.of(endedHouseDevice));
        device.setHouseDevices(Set.of(houseDevice, endedHouseDevice));

        DeviceDetailDTO dto = mapper.map(device);

        assertThat(dto).isNotNull();
        assertThat(dto.getId()).isEqualTo(0L);
        assertThat(dto.getHouse()).isEqualTo(new DeviceDetailHouseDTO(0L, house.getName()));
        assertThat(dto.getNickname()).isEqualTo(device.getNickName());
        assertThat(dto.getModel()).isEqualTo(device.getModel());
        assertThat(dto.getPowerStatus()).isEqualTo(PowerStatusDto.ON);
    }

    @Test
    void notAssignedTest() {
        AppUser user = new AppUser("email", "hash", "firstName", "lastName", "123456789", Gender.FEMALE, UserRole.APP_USER);
        Device device = new Device(user, "Nice Device", "Model 123", PowerStatus.ON);

        DeviceDetailDTO dto = mapper.map(device);

        assertThat(dto).isNotNull();
        assertThat(dto.getId()).isEqualTo(0L);
        assertThat(dto.getHouse()).isEqualTo(null);
        assertThat(dto.getNickname()).isEqualTo(device.getNickName());
        assertThat(dto.getModel()).isEqualTo(device.getModel());
        assertThat(dto.getPowerStatus()).isEqualTo(PowerStatusDto.ON);
    }

}