package cz.muni.fi.pa165.project.config;

import cz.muni.fi.pa165.project.BaseServiceLayerTest;
import cz.muni.fi.pa165.project.dto.HouseDetailConsumptionEventDTO;
import cz.muni.fi.pa165.project.dto.HouseDetailConsumptionEventListDeviceDTO;
import cz.muni.fi.pa165.project.entity.AppUser;
import cz.muni.fi.pa165.project.entity.ConsumptionEvent;
import cz.muni.fi.pa165.project.entity.Device;
import cz.muni.fi.pa165.project.enums.Gender;
import cz.muni.fi.pa165.project.enums.PowerStatus;
import cz.muni.fi.pa165.project.enums.UserRole;
import org.junit.jupiter.api.Test;
import org.mockito.Mockito;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;

import java.time.LocalDateTime;
import java.util.List;

import static org.assertj.core.api.Assertions.assertThat;

/**
 * @author Aneta Čadová | 485025
 */
class HouseDetailConsumptionEventMapperTest extends BaseServiceLayerTest {
    @Autowired
    private HouseDetailConsumptionEventMapper houseDetailConsumptionEventMapper;

    @Test
    void mapConsumptionEventToHouseDetailConsumptionEventDtoTest() {
        AppUser user = new AppUser("email", "hash", "firstName", "lastName", "123456789", Gender.FEMALE, UserRole.APP_USER);
        Device device = new Device(user, "nickName", "model", PowerStatus.ON);
        ConsumptionEvent consumptionEvent = new ConsumptionEvent(device, LocalDateTime.now(), 25.3);

        HouseDetailConsumptionEventDTO houseDetailConsumptionEventDto = houseDetailConsumptionEventMapper.map(consumptionEvent);

        assertThat(houseDetailConsumptionEventDto).isNotNull();
        assertThat(houseDetailConsumptionEventDto.getId()).isEqualTo(0L);
        assertThat(houseDetailConsumptionEventDto.getTimestamp()).isEqualTo(consumptionEvent.getTimestamp());
        assertThat(houseDetailConsumptionEventDto.getDevice()).isEqualTo(new HouseDetailConsumptionEventListDeviceDTO(0L, device.getNickName()));
    }

    @Test
    void mapDeviceToHouseDetailConsumptionEventListDeviceDTOTest() {
        Device device = new Device(null, "nickName", "model", PowerStatus.ON);

        HouseDetailConsumptionEventListDeviceDTO houseDetailConsumptionEventListDeviceDTO = houseDetailConsumptionEventMapper.map(device);

        assertThat(houseDetailConsumptionEventListDeviceDTO).isNotNull();
        assertThat(houseDetailConsumptionEventListDeviceDTO.getId()).isEqualTo(0L);
        assertThat(houseDetailConsumptionEventListDeviceDTO.getNickname()).isEqualTo(device.getNickName());
    }

    @Test
    void mapConsumptionEventPageToHouseDetailConsumptionEventDtoTestPage() {
        AppUser user = new AppUser("email", "hash", "firstName", "lastName", "123456789", Gender.FEMALE, UserRole.APP_USER);
        Device device = new Device(user, "nickName", "model", PowerStatus.ON);
        Pageable pageable = Mockito.mock(Pageable.class);
        ConsumptionEvent consumptionEvent = new ConsumptionEvent(device, LocalDateTime.now(), 25.3);
        Page<ConsumptionEvent> consumptionEventPage = new PageImpl<>(List.of(consumptionEvent), pageable, 2);

        Page<HouseDetailConsumptionEventDTO> houseDetailConsumptionEventDtoPage = houseDetailConsumptionEventMapper.map(consumptionEventPage);

        assertThat(houseDetailConsumptionEventDtoPage).isNotNull();
    }
}

