package cz.muni.fi.pa165.project.config;

import cz.muni.fi.pa165.project.BaseServiceLayerTest;
import cz.muni.fi.pa165.project.dto.HouseListDTO;
import cz.muni.fi.pa165.project.entity.AppUser;
import cz.muni.fi.pa165.project.entity.Device;
import cz.muni.fi.pa165.project.entity.House;
import cz.muni.fi.pa165.project.entity.HouseDevice;
import cz.muni.fi.pa165.project.enums.Gender;
import cz.muni.fi.pa165.project.enums.PowerStatus;
import cz.muni.fi.pa165.project.enums.UserRole;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;

import java.time.LocalDateTime;
import java.util.Set;

import static org.assertj.core.api.Assertions.assertThat;

/**
 * @author Daniela Hostačná | 485432
 */
class HouseListMapperTest extends BaseServiceLayerTest {
    @Autowired
    private HouseListMapper mapper;

    @Test
    void mapEntityToDto() {
        AppUser user = new AppUser("email", "hash", "firstName", "lastName", "123456789", Gender.FEMALE, UserRole.APP_USER);
        House house = new House(user, "FI MUNI", "Botanicka 68A", "Brno", "60200", "CZ");
        House previousHouse = new House(user, "FIT VUT", "Sever Brna", "Brno", "60200", "CZ");
        Device device1 = new Device(user, "Nice Device", "Model 123", PowerStatus.ON);
        Device device2 = new Device(user, "Pretty Device", "Model 123", PowerStatus.ON);
        HouseDevice houseDevice1 = new HouseDevice(house, device1, LocalDateTime.now().minusDays(1), null);
        HouseDevice houseDevice2 = new HouseDevice(house, device2, LocalDateTime.now().minusDays(5), null);
        HouseDevice endedHouseDevice = new HouseDevice(previousHouse, device1, LocalDateTime.now().minusDays(10), LocalDateTime.now().minusDays(1).minusMinutes(1));
        house.setHouseDevices(Set.of(houseDevice1, houseDevice2));
        previousHouse.setHouseDevices(Set.of(endedHouseDevice));
        device1.setHouseDevices(Set.of(houseDevice1, endedHouseDevice));

        HouseListDTO dto = mapper.map(house);

        assertThat(dto).isNotNull();
        assertThat(dto.getId()).isEqualTo(0L);
        assertThat(dto.getName()).isEqualTo(house.getName());
        assertThat(dto.getStreet()).isEqualTo(house.getStreet());
        assertThat(dto.getCity()).isEqualTo(house.getCity());
        assertThat(dto.getZip()).isEqualTo(house.getZip());
        assertThat(dto.getCountry()).isEqualTo(house.getCountry());
        assertThat(dto.getAssignedDeviceCount()).isEqualTo(2);
    }

}