package cz.muni.fi.pa165.project.facade;

import cz.muni.fi.pa165.project.BaseServiceLayerTest;
import cz.muni.fi.pa165.project.TestServiceConfiguration;
import cz.muni.fi.pa165.project.config.DistributorConsumptionEventListMapper;
import cz.muni.fi.pa165.project.dto.DistributorConsumptionEventListDTO;
import cz.muni.fi.pa165.project.entity.AppUser;
import cz.muni.fi.pa165.project.entity.ConsumptionEvent;
import cz.muni.fi.pa165.project.entity.Device;
import cz.muni.fi.pa165.project.enums.Gender;
import cz.muni.fi.pa165.project.enums.PowerStatus;
import cz.muni.fi.pa165.project.enums.UserRole;
import cz.muni.fi.pa165.project.service.AppUserService;
import cz.muni.fi.pa165.project.service.ConsumptionEventService;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.mockito.Mockito;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.context.annotation.Import;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;

import java.time.LocalDateTime;
import java.util.List;

import static org.assertj.core.api.AssertionsForInterfaceTypes.assertThat;

/**
 * @author Aneta Čadová | 485025
 */
class ConsumptionEventFacadeTest extends BaseServiceLayerTest {
    @MockBean
    ConsumptionEventService consumptionEventService;

    @MockBean
    AppUserService appUserService;

    @MockBean
    DistributorConsumptionEventListMapper distributorConsumptionEventListMapper;

    @Autowired
    private ConsumptionEventFacadeImpl consumptionEventFacade;

    private final Pageable pageable = PageRequest.of(0, 2);
    private final AppUser user = new AppUser("email1", "hash", "firstName", "lastName", "123456789", Gender.FEMALE, UserRole.APP_USER);
    private final Device device = new Device(user, "nickName", "model", PowerStatus.ON);

    private final ConsumptionEvent consumptionEvent = new ConsumptionEvent(device, LocalDateTime.now(), 25.3);
    private final Page<ConsumptionEvent> consumptionEventPage = new PageImpl<>(List.of(consumptionEvent), pageable, 2);
    private final DistributorConsumptionEventListDTO distributorConsumptionEventListDTO = new DistributorConsumptionEventListDTO(
            1L, consumptionEvent.getTimestamp(), consumptionEvent.getPowerConsumption(),
            consumptionEvent.getDevice().getOwner().getFirstName() + " " + consumptionEvent.getDevice().getOwner().getLastName());

    private final Page<DistributorConsumptionEventListDTO> distributorConsumptionEventListDTOPage = new PageImpl<>(List.of(distributorConsumptionEventListDTO), pageable, 2);


    @BeforeEach
    void setup() {
        user.setId(1L);
        consumptionEvent.setId(1L);
    }

    @Test
    void getAllTest() {
        Mockito.when(appUserService.findById(user.getId())).thenReturn(user);
        Mockito.when(consumptionEventService.findAll(PageRequest.of(0, 2))).thenReturn(consumptionEventPage);
        Mockito.when(distributorConsumptionEventListMapper.map(consumptionEventPage)).thenReturn(distributorConsumptionEventListDTOPage);

        var allConsumptionEventsTest = consumptionEventFacade.getAll(pageable);
        assertThat(allConsumptionEventsTest).isNotNull();
        assertThat(allConsumptionEventsTest).isEqualTo(distributorConsumptionEventListDTOPage);
    }
}
