package cz.muni.fi.pa165.project.facade;

import cz.muni.fi.pa165.project.BaseServiceLayerTest;
import cz.muni.fi.pa165.project.config.DeviceCreateMapper;
import cz.muni.fi.pa165.project.config.DeviceDetailConsumptionEventMapper;
import cz.muni.fi.pa165.project.config.DeviceDetailMapper;
import cz.muni.fi.pa165.project.config.DeviceListMapper;
import cz.muni.fi.pa165.project.dto.DeviceAssignHouseDTO;
import cz.muni.fi.pa165.project.dto.DeviceCreateDTO;
import cz.muni.fi.pa165.project.dto.DeviceDetailConsumptionEventDTO;
import cz.muni.fi.pa165.project.dto.DeviceDetailDTO;
import cz.muni.fi.pa165.project.dto.DeviceDetailHouseDTO;
import cz.muni.fi.pa165.project.dto.DeviceListDTO;
import cz.muni.fi.pa165.project.dto.DeviceListHouseDTO;
import cz.muni.fi.pa165.project.dto.HouseNameDTO;
import cz.muni.fi.pa165.project.entity.AppUser;
import cz.muni.fi.pa165.project.entity.ConsumptionEvent;
import cz.muni.fi.pa165.project.entity.Device;
import cz.muni.fi.pa165.project.entity.House;
import cz.muni.fi.pa165.project.entity.HouseDevice;
import cz.muni.fi.pa165.project.enums.Gender;
import cz.muni.fi.pa165.project.enums.PowerStatus;
import cz.muni.fi.pa165.project.enums.PowerStatusDto;
import cz.muni.fi.pa165.project.enums.UserRole;
import cz.muni.fi.pa165.project.service.AppUserService;
import cz.muni.fi.pa165.project.service.ConsumptionEventService;
import cz.muni.fi.pa165.project.service.DeviceService;
import cz.muni.fi.pa165.project.service.HouseDeviceService;
import cz.muni.fi.pa165.project.service.HouseService;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Disabled;
import org.junit.jupiter.api.Test;
import org.mockito.Mockito;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;

import java.time.LocalDateTime;
import java.util.List;

import static org.assertj.core.api.AssertionsForInterfaceTypes.assertThat;
import static org.junit.jupiter.api.Assertions.assertThrows;
import static org.mockito.ArgumentMatchers.eq;
import static org.mockito.Mockito.times;

/**
 * @author Aneta Čadová | 485025
 */
class DeviceFacadeTest extends BaseServiceLayerTest {
    @MockBean
    DeviceService deviceService;

    @MockBean
    AppUserService appUserService;

    @MockBean
    ConsumptionEventService consumptionEventService;

    @MockBean
    HouseService houseService;

    @MockBean
    HouseDeviceService houseDeviceService;

    @MockBean
    DeviceListMapper deviceListMapper;

    @MockBean
    DeviceDetailMapper deviceDetailMapper;

    @MockBean
    DeviceCreateMapper deviceCreateMapper;

    @MockBean
    DeviceDetailConsumptionEventMapper deviceDetailConsumptionEventMapper;

    @Autowired
    DeviceFacadeImpl deviceFacade;

    private final Pageable pageable = PageRequest.of(0, 2);
    private final AppUser user = new AppUser("email", "hash", "firstName", "lastName", "123456789", Gender.FEMALE, UserRole.APP_USER);
    private final Device device = new Device(user, "nickName", "model", PowerStatus.ON);
    private final House house = new House(user, "FI MUNI", "Brno", "CZ", "Botanicka 68A", "60200");

    private final ConsumptionEvent consumptionEvent = new ConsumptionEvent(device, LocalDateTime.now(), 25.3);
    private final Page<ConsumptionEvent> consumptionEventPage = new PageImpl<>(List.of(consumptionEvent), pageable, 2);
    private final DeviceListHouseDTO deviceListHouseDTO = new DeviceListHouseDTO(1L, house.getName());

    private final DeviceListDTO deviceListDTO = new DeviceListDTO(1L, device.getNickName(), device.getModel(), PowerStatusDto.ON, deviceListHouseDTO);
    private final Page<Device> devicePage = new PageImpl<>(List.of(device), pageable, 2);
    private final Page<DeviceListDTO> deviceListDTOPage = new PageImpl<>(List.of(deviceListDTO), pageable, 2);

    @BeforeEach
    void setup() {
        user.setId(1L);
        device.setId(1L);
        house.setId(1L);
        consumptionEvent.setId(1L);
    }

    @Test
    void getDevicesByUserTest() {
        Mockito.when(appUserService.findByEmail(user.getEmail())).thenReturn(user);
        Mockito.when(deviceService.findDevicesForUser(user, pageable)).thenReturn(devicePage);
        Mockito.when(deviceListMapper.map(devicePage)).thenReturn(deviceListDTOPage);

        var deviceListDTOPageTest = deviceFacade.getDevicesByUser(user.getEmail(), PageRequest.of(0, 2));

        assertThat(deviceListDTOPageTest).isNotNull();
        assertThat(deviceListDTOPageTest).isEqualTo(deviceListDTOPage);
    }

    @Test
    void getDevicesByUserNullUserExceptionTest() {
        Mockito.when(appUserService.findByEmail(user.getEmail())).thenReturn(null);
        assertThrows(IllegalArgumentException.class, () -> deviceFacade.getDevicesByUser(user.getEmail(), PageRequest.of(0, 2)), "Requester user does not exist");
    }

    @Test
    void getDeviceTest() {
        DeviceDetailHouseDTO deviceDetailHouseDTO = new DeviceDetailHouseDTO(house.getId(), house.getName());
        DeviceDetailDTO deviceDetailDTO = new DeviceDetailDTO(1L, device.getNickName(), device.getModel(), PowerStatusDto.ON, deviceDetailHouseDTO);

        Mockito.when(appUserService.findByEmail(user.getEmail())).thenReturn(user);
        Mockito.when(deviceService.findById(device.getId())).thenReturn(device);
        Mockito.when(deviceDetailMapper.map(device)).thenReturn(deviceDetailDTO);

        var deviceDetailDTOPageTest = deviceFacade.getDevice(user.getEmail(), device.getId());

        assertThat(deviceDetailDTOPageTest).isNotNull();
        assertThat(deviceDetailDTOPageTest).isEqualTo(deviceDetailDTO);
    }

    @Test
    void getDeviceTestNullUserExceptionTest() {
        Mockito.when(appUserService.findByEmail(user.getEmail())).thenReturn(null);
        assertThrows(IllegalArgumentException.class, () -> deviceFacade.getDevicesByUser(user.getEmail(), PageRequest.of(0, 2)), "User does not exist.");
    }

    @Test
    void getDeviceTestNullDeviceExceptionTest() {
        Mockito.when(appUserService.findByEmail(user.getEmail())).thenReturn(user);
        Mockito.when(deviceService.findById(device.getId())).thenReturn(null);

        assertThrows(IllegalArgumentException.class, () -> deviceFacade.getDevice(user.getEmail(), device.getId()), "Device does not exist.");
    }

    @Test
    void getDeviceTestUserNotOwnerExceptionTest() {
        Device deviceIncorrectOwner = new Device(new AppUser(), "nickName1", "model1", PowerStatus.OFF);
        deviceIncorrectOwner.setId(2L);

        Mockito.when(appUserService.findByEmail(user.getEmail())).thenReturn(user);
        Mockito.when(deviceService.findById(deviceIncorrectOwner.getId())).thenReturn(deviceIncorrectOwner);

        assertThrows(IllegalArgumentException.class, () -> deviceFacade.getDevice(user.getEmail(), deviceIncorrectOwner.getId()), "The user is not owner of the device.");
    }

    @Test
    void createDeviceTest() {
        Device createdDevice = new Device(null, "nickName", "model", PowerStatus.OFF);
        createdDevice.setId(2L);
        DeviceCreateDTO deviceCreateDTO = new DeviceCreateDTO(createdDevice.getNickName(), createdDevice.getModel());

        Mockito.when(appUserService.findByEmail(user.getEmail())).thenReturn(user);
        Mockito.when(deviceCreateMapper.map(deviceCreateDTO)).thenReturn(createdDevice);
        Mockito.when(deviceService.create(createdDevice)).thenReturn(createdDevice.getId());
        Mockito.when(deviceService.findById(createdDevice.getId())).thenReturn(createdDevice);

        var createdDeviceId = deviceFacade.createDevice(user.getEmail(), deviceCreateDTO);
        assertThat(createdDeviceId).isNotNull();
        assertThat(createdDeviceId).isEqualTo(createdDevice.getId());
        assertThat(deviceService.findById(createdDeviceId).getOwner()).isEqualTo(user);
    }

    @Test
    void getConsumptionEventsByDeviceTest() {
        DeviceDetailConsumptionEventDTO deviceDetailConsumptionEventDTO = new DeviceDetailConsumptionEventDTO(consumptionEvent.getId(), new HouseNameDTO(house.getId(), house.getName()), consumptionEvent.getTimestamp(), consumptionEvent.getPowerConsumption());
        Page<DeviceDetailConsumptionEventDTO> deviceDetailConsumptionEventDTOPage = new PageImpl<>(List.of(deviceDetailConsumptionEventDTO), pageable, 2);
        Mockito.when(appUserService.findByEmail(user.getEmail())).thenReturn(user);
        Mockito.when(deviceService.findById(device.getId())).thenReturn(device);
        Mockito.when(consumptionEventService.findConsumptionEventsForDevice(device, pageable)).thenReturn(consumptionEventPage);
        Mockito.when(deviceDetailConsumptionEventMapper.map(consumptionEventPage)).thenReturn(deviceDetailConsumptionEventDTOPage);

        var deviceDetailConsumptionEventDTOTest = deviceFacade.getConsumptionEventsByDevice(user.getEmail(), device.getId(), PageRequest.of(0, 2));
        assertThat(deviceDetailConsumptionEventDTOTest).isNotNull();
        assertThat(deviceDetailConsumptionEventDTOTest).isEqualTo(deviceDetailConsumptionEventDTOPage);
    }

    @Test
    @Disabled("Removed assign date from device facade, possible errors")
    void assignDeviceToHouseTest() {
        DeviceAssignHouseDTO assignHouseDTO = new DeviceAssignHouseDTO(LocalDateTime.now());
        HouseDevice houseDevice = new HouseDevice(house, device, assignHouseDTO.getAssignDate(), null);
        houseDevice.setId(1L);

        Mockito.when(appUserService.findByEmail(user.getEmail())).thenReturn(user);
        Mockito.when(deviceService.findById(device.getId())).thenReturn(device);
        Mockito.when(houseService.findById(house.getId())).thenReturn(house);
        Mockito.when(houseDeviceService.create(houseDevice)).thenReturn(houseDevice.getId());

        var assignmentId = deviceFacade.assignDeviceToHouse(user.getEmail(), device.getId(), house.getId());
        assertThat(assignmentId).isNotNull();
        assertThat(assignmentId).isEqualTo(houseDevice.getId());
    }

    @Test
    void unAssignDeviceFromHouseTest() {
        DeviceAssignHouseDTO assignHouseDTO = new DeviceAssignHouseDTO(LocalDateTime.now());
        HouseDevice houseDevice = new HouseDevice(house, device, assignHouseDTO.getAssignDate(), null);
        houseDevice.setId(1L);

        Mockito.when(appUserService.findByEmail(user.getEmail())).thenReturn(user);
        Mockito.when(deviceService.findById(device.getId())).thenReturn(device);
        Mockito.when(houseService.findById(house.getId())).thenReturn(house);

        deviceFacade.unAssignDeviceFromHouse(user.getEmail(), device.getId(), house.getId());

        Mockito.verify(houseDeviceService, times(1)).closeAssignmentFor(eq(device), eq(house));
    }

    @Test
    void deleteDeviceTest() {
        Mockito.when(appUserService.findByEmail(user.getEmail())).thenReturn(user);
        Mockito.when(deviceService.findById(device.getId())).thenReturn(device);

        deviceFacade.deleteDevice(user.getEmail(), device.getId());

        Mockito.verify(deviceService, times(1)).delete(eq(device));
    }
}
