package cz.muni.fi.pa165.project.service;

import cz.muni.fi.pa165.project.BaseServiceLayerTest;
import cz.muni.fi.pa165.project.entity.AppUser;
import cz.muni.fi.pa165.project.enums.Gender;
import cz.muni.fi.pa165.project.enums.UserRole;
import cz.muni.fi.pa165.project.repository.AppUserRepository;
import org.junit.jupiter.api.Test;
import org.mockito.Mockito;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.mock.mockito.MockBean;

import java.util.ArrayList;
import java.util.List;

import static org.assertj.core.api.Assertions.assertThat;
import static org.mockito.ArgumentMatchers.eq;
import static org.mockito.Mockito.times;

/**
 * @author Aneta Čadová | 485025
 */
class AppUserServiceTest extends BaseServiceLayerTest {

    @MockBean
    private AppUserRepository appUserRepository;

    @Autowired
    private AppUserServiceImpl appUserService;

    private final AppUser user1 = new AppUser("email1", "hash", "firstName", "lastName", "123456789", Gender.FEMALE, UserRole.APP_USER);
    private final AppUser user2 = new AppUser("email2", "hash", "firstName", "lastName", "123456789", Gender.FEMALE, UserRole.APP_USER);

    @Test
    void findByIdTest() {
        Mockito.when(appUserRepository.findById(1L)).thenReturn(java.util.Optional.of(user1));

        var testUser = appUserService.findById(1);
        assertThat(testUser).isNotNull();
        assertThat(testUser).isEqualTo(user1);
    }

    @Test
    void findAllTest() {

        List<AppUser> appUserList = new ArrayList<>();
        appUserList.add(user1);
        appUserList.add(user2);
        Mockito.when(appUserRepository.findAll()).thenReturn(appUserList);

        var testUserList = appUserService.findAll();
        assertThat(testUserList).isNotNull();
        assertThat(testUserList).containsExactly(user1, user2);
    }

    @Test
    void createTest() {
        user1.setId(1L);
        Mockito.when(appUserRepository.save(user1)).thenReturn(user1);
        var userId = appUserService.create(user1);

        assertThat(userId).isNotNull();
        assertThat(userId).isEqualTo(user1.getId());
    }

    @Test
    void updateTest() {
        appUserService.update(user1);
        Mockito.verify(appUserRepository, times(1)).saveAndFlush(eq(user1));
    }

    @Test
    void deleteTest() {
        appUserService.delete(user1);
        Mockito.verify(appUserRepository, times(1)).delete(eq(user1));
    }
}