package cz.muni.fi.pa165.project.service;

import cz.muni.fi.pa165.project.BaseServiceLayerTest;
import cz.muni.fi.pa165.project.entity.ConsumptionEvent;
import cz.muni.fi.pa165.project.entity.Device;
import cz.muni.fi.pa165.project.repository.ConsumptionEventRepository;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.mockito.Mockito;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

import static org.assertj.core.api.Assertions.assertThat;
import static org.mockito.ArgumentMatchers.eq;
import static org.mockito.Mockito.times;

/**
 * @author Aneta Čadová | 485025
 */
class ConsumptionEventServiceTest extends BaseServiceLayerTest {
    @MockBean
    private ConsumptionEventRepository consumptionEventRepository;

    @Autowired
    private ConsumptionEventServiceImpl consumptionEventService;

    private final Pageable pageable = Mockito.mock(Pageable.class);

    private final List<ConsumptionEvent> consumptionEventList = new ArrayList<>();
    private final ConsumptionEvent consumptionEvent1 = new ConsumptionEvent();
    private final ConsumptionEvent consumptionEvent2 = new ConsumptionEvent();
    private final List<Long> deviceIdsList = new ArrayList<>();

    private Page<ConsumptionEvent> consumptionEventPage;

    @BeforeEach
    void setup() {
        consumptionEventList.add(consumptionEvent1);
        consumptionEventList.add(consumptionEvent2);
        consumptionEventPage = new PageImpl<>(consumptionEventList, pageable, 2);

        deviceIdsList.add(1L);
        deviceIdsList.add(2L);
    }

    @Test
    void findConsumptionEventsForDevicesTest() {
        Mockito.when(consumptionEventRepository.findAllByDeviceIds(deviceIdsList, pageable)).thenReturn(consumptionEventPage);

        Page<ConsumptionEvent> consumptionEvents = consumptionEventService.findConsumptionEventsForDevices(deviceIdsList, pageable);
        var consumptionEvenTestList = consumptionEvents.get().collect(Collectors.toList());

        assertThat(consumptionEvenTestList).hasSize(2);
        assertThat(consumptionEvenTestList).containsExactly(consumptionEvent1, consumptionEvent2);
    }

    @Test
    void findConsumptionEventsForDeviceTest() {
        Device device = new Device();
        Mockito.when(consumptionEventRepository.findAllByDevice(device, pageable)).thenReturn(consumptionEventPage);

        Page<ConsumptionEvent> consumptionEvents = consumptionEventService.findConsumptionEventsForDevice(device, pageable);
        var consumptionEvenTestList = consumptionEvents.get().collect(Collectors.toList());

        assertThat(consumptionEvenTestList).hasSize(2);
        assertThat(consumptionEvenTestList).containsExactly(consumptionEvent1, consumptionEvent2);

    }

    @Test
    void findAllEventsForUserIdTest() {
        Mockito.when(consumptionEventRepository.findAll(pageable)).thenReturn(consumptionEventPage);

        Page<ConsumptionEvent> consumptionEvents = consumptionEventService.findAll(pageable);
        var consumptionEvenTestList = consumptionEvents.get().collect(Collectors.toList());

        assertThat(consumptionEvenTestList).hasSize(2);
        assertThat(consumptionEvenTestList).containsExactly(consumptionEvent1, consumptionEvent2);

    }

    @Test
    void deleteAllByDeviceIdsTest() {
        consumptionEventService.deleteAllByDeviceIds(deviceIdsList);
        Mockito.verify(consumptionEventRepository, times(1)).deleteAllByDeviceIds(eq(deviceIdsList));
    }
}
