package cz.muni.fi.pa165.project.service;

import cz.muni.fi.pa165.project.BaseServiceLayerTest;
import cz.muni.fi.pa165.project.entity.AppUser;
import cz.muni.fi.pa165.project.entity.Device;
import cz.muni.fi.pa165.project.entity.House;
import cz.muni.fi.pa165.project.entity.HouseDevice;
import cz.muni.fi.pa165.project.enums.PowerStatus;
import cz.muni.fi.pa165.project.repository.HouseDeviceRepository;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.mockito.Mockito;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.mock.mockito.MockBean;

import java.time.Clock;
import java.time.Instant;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.temporal.ChronoUnit;
import java.util.ArrayList;
import java.util.List;

import static org.assertj.core.api.Assertions.assertThat;
import static org.junit.jupiter.api.Assertions.assertThrows;

/**
 * @author Aneta Čadová | 485025
 */
class HouseDeviceServiceTest extends BaseServiceLayerTest {

    @MockBean
    private HouseDeviceRepository houseDeviceRepository;
    @MockBean
    private Clock clock;


    @Autowired
    private HouseDeviceServiceImpl houseDeviceService;

    private final AppUser user = new AppUser();
    private final Device device = new Device(user, "nickName", "model", PowerStatus.ON);
    private final House house = new House(user, "FI MUNI", "Brno", "CZ", "Botanicka 68A", "60200");
    private final HouseDevice houseDevice = new HouseDevice(house, device, LocalDateTime.now().minusDays(2), LocalDateTime.now().minusDays(1));
    private final List<HouseDevice> houseDeviceList = new ArrayList<>();

    @BeforeEach
    void setup() {
        houseDevice.setId(1L);
        houseDeviceList.add(houseDevice);
    }

    @Test
    void createTest() {
        Mockito.when(houseDeviceRepository.save(houseDevice)).thenReturn(houseDevice);
        var assignmentId = houseDeviceService.create(houseDevice);

        assertThat(assignmentId).isNotNull();
        assertThat(assignmentId).isEqualTo(houseDevice.getId());
    }

    @Test
    void closeAssignmentForOneAssignmentTest() {
        Mockito.when(houseDeviceRepository.findOpenByDeviceAndHouse(device, house)).thenReturn(houseDeviceList);
        var fixedClock = Clock.fixed(Instant.now(), ZoneId.systemDefault());
        Mockito.when(clock.instant()).thenReturn(fixedClock.instant());
        Mockito.when(clock.getZone()).thenReturn(fixedClock.getZone());

        houseDeviceService.closeAssignmentFor(device, house);

        var houseDevices = houseDeviceRepository.findOpenByDeviceAndHouse(device, house);
        assertThat(ChronoUnit.SECONDS.between(houseDevices.get(0).getValidTo(), LocalDateTime.now(fixedClock))).isLessThan(2);
    }

    @Test
    void closeAssignmentForMoreActiveExceptionTest() {
        houseDeviceList.add(new HouseDevice());
        Mockito.when(houseDeviceRepository.findOpenByDeviceAndHouse(device, house)).thenReturn(houseDeviceList);

        assertThrows(IllegalArgumentException.class, () -> houseDeviceService.closeAssignmentFor(device, house), "More active assignments.");
    }

    @Test
    void closeAssignmentForNoActiveExceptionTest() {
        List<HouseDevice> empty = new ArrayList<>();
        Mockito.when(houseDeviceRepository.findOpenByDeviceAndHouse(device, house)).thenReturn(empty);

        assertThrows(IllegalArgumentException.class, () -> houseDeviceService.closeAssignmentFor(device, house), "No active assignment to close");
    }
}
